#!/usr/bin/python
# Copyright (c) 2018, David Kainz <dkainz@mgit.at> <dave.jokain@gmx.at>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: openssh_keypair
author: "David Kainz (@lolcube)"
short_description: Generate OpenSSH private and public keys
description:
  - This module allows one to (re)generate OpenSSH private and public keys. It uses ssh-keygen to generate keys. One can generate
    V(rsa), V(dsa), V(rsa1), V(ed25519) or V(ecdsa) private keys.
requirements:
  - ssh-keygen (if O(backend=openssh))
  - cryptography >= 3.3 (if O(backend=cryptography))
  - bcrypt (if O(backend=cryptography) and O(passphrase) is used)
extends_documentation_fragment:
  - ansible.builtin.files
  - community.crypto._attributes
  - community.crypto._attributes.files
attributes:
  check_mode:
    support: full
  diff_mode:
    support: full
  safe_file_operations:
    support: full
  idempotent:
    support: partial
    details:
      - The module is not idempotent if O(force=true) or O(regenerate=always).
options:
  state:
    description:
      - Whether the private and public keys should exist or not, taking action if the state is different from what is stated.
    type: str
    default: present
    choices: [present, absent]
  size:
    description:
      - 'Specifies the number of bits in the private key to create. For RSA keys, the minimum size is 1024 bits and the default
        is 4096 bits. Generally, 2048 bits is considered sufficient. DSA keys must be exactly 1024 bits as specified by FIPS
        186-2. For ECDSA keys, size determines the key length by selecting from one of three elliptic curve sizes: 256, 384
        or 521 bits. Attempting to use bit lengths other than these three values for ECDSA keys will cause this module to
        fail. Ed25519 keys have a fixed length and the size will be ignored.'
    type: int
  type:
    description:
      - The algorithm used to generate the SSH private key. V(rsa1) is for protocol version 1. V(rsa1) is deprecated and may
        not be supported by every version of ssh-keygen.
    type: str
    default: rsa
    choices: ['rsa', 'dsa', 'rsa1', 'ecdsa', 'ed25519']
  force:
    description:
      - Should the key be regenerated even if it already exists.
    type: bool
    default: false
  path:
    description:
      - Name of the files containing the public and private key. The file containing the public key will have the extension
        C(.pub).
    type: path
    required: true
  comment:
    description:
      - Provides a new comment to the public key.
    type: str
  passphrase:
    description:
      - Passphrase used to decrypt an existing private key or encrypt a newly generated private key.
      - Passphrases are not supported for O(type=rsa1).
      - Can only be used when O(backend=cryptography), or when O(backend=auto) and a required C(cryptography) version is installed.
        Note that depending on the cryptography version, the C(bcrypt) package is also needed.
    type: str
    version_added: 1.7.0
  private_key_format:
    description:
      - Used when O(backend=cryptography) to select a format for the private key at the provided O(path).
      - When set to V(auto) this module will match the key format of the installed OpenSSH version.
      - For OpenSSH < 7.8 private keys will be in PKCS1 format except ed25519 keys which will be in OpenSSH format.
      - For OpenSSH >= 7.8 all private key types will be in the OpenSSH format.
      - Using this option when O(regenerate=partial_idempotence) or O(regenerate=full_idempotence) will cause a new keypair
        to be generated if the private key's format does not match the value of O(private_key_format). This module will not
        however convert existing private keys between formats.
    type: str
    default: auto
    choices:
      - auto
      - pkcs1
      - pkcs8
      - ssh
    version_added: 1.7.0
  backend:
    description:
      - Selects between the V(cryptography) library or the OpenSSH binary V(opensshbin).
      - V(auto) will default to V(opensshbin) unless the OpenSSH binary is not installed or when using O(passphrase).
    type: str
    default: auto
    choices:
      - auto
      - cryptography
      - opensshbin
    version_added: 1.7.0
  regenerate:
    description:
      - Allows to configure in which situations the module is allowed to regenerate private keys. The module will always generate
        a new key if the destination file does not exist.
      - By default, the key will be regenerated when it does not match the module's options, except when the key cannot be
        read or the passphrase does not match. Please note that this B(changed) for Ansible 2.10. For Ansible 2.9, the behavior
        was as if V(full_idempotence) is specified.
      - If set to V(never), the module will fail if the key cannot be read or the passphrase is not matching, and will never
        regenerate an existing key.
      - If set to V(fail), the module will fail if the key does not correspond to the module's options.
      - If set to V(partial_idempotence), the key will be regenerated if it does not conform to the module's options. The
        key is B(not) regenerated if it cannot be read (broken file), the key is protected by an unknown passphrase, or when
        they key is not protected by a passphrase, but a passphrase is specified.
      - If set to V(full_idempotence), the key will be regenerated if it does not conform to the module's options. This is
        also the case if the key cannot be read (broken file), the key is protected by an unknown passphrase, or when they
        key is not protected by a passphrase, but a passphrase is specified. Make sure you have a B(backup) when using this
        option!
      - If set to V(always), the module will always regenerate the key. This is equivalent to setting O(force) to V(true).
      - Note that adjusting the comment and the permissions can be changed without regeneration. Therefore, even for V(never),
        the task can result in changed.
    type: str
    choices:
      - never
      - fail
      - partial_idempotence
      - full_idempotence
      - always
    default: partial_idempotence
    version_added: '1.0.0'
notes:
  - In case the ssh key is broken or password protected, the module will fail. Set the O(force) option to V(true) if you want
    to regenerate the keypair.
  - In the case a custom O(mode), O(group), O(owner), or other file attribute is provided it will be applied to both key files.
"""

EXAMPLES = r"""
---
- name: Generate an OpenSSH keypair with the default values (4096 bits, rsa)
  community.crypto.openssh_keypair:
    path: /tmp/id_ssh_rsa

- name: Generate an OpenSSH keypair with the default values (4096 bits, rsa) and encrypted private key
  community.crypto.openssh_keypair:
    path: /tmp/id_ssh_rsa
    passphrase: super_secret_password

- name: Generate an OpenSSH rsa keypair with a different size (2048 bits)
  community.crypto.openssh_keypair:
    path: /tmp/id_ssh_rsa
    size: 2048

- name: Force regenerate an OpenSSH keypair if it already exists
  community.crypto.openssh_keypair:
    path: /tmp/id_ssh_rsa
    force: true

- name: Regenerate SSH keypair only if format or options mismatch
  community.crypto.openssh_keypair:
    path: /home/devops/.ssh/id_ed25519
    type: ed25519
    regenerate: full_idempotence
    private_key_format: ssh

- name: Generate an OpenSSH keypair with a different algorithm (dsa)
  community.crypto.openssh_keypair:
    path: /tmp/id_ssh_dsa
    type: dsa
"""

RETURN = r"""
size:
  description: Size (in bits) of the SSH private key.
  returned: changed or success
  type: int
  sample: 4096
type:
  description: Algorithm used to generate the SSH private key.
  returned: changed or success
  type: str
  sample: rsa
filename:
  description: Path to the generated SSH private key file.
  returned: changed or success
  type: str
  sample: /tmp/id_ssh_rsa
fingerprint:
  description: The fingerprint of the key.
  returned: changed or success
  type: str
  sample: SHA256:r4YCZxihVjedH2OlfjVGI6Y5xAYtdCwk8VxKyzVyYfM
public_key:
  description: The public key of the generated SSH private key.
  returned: changed or success
  type: str
  sample: ssh-rsa AAAAB3Nza(...omitted...)veL4E3Xcw==
comment:
  description: The comment of the generated key.
  returned: changed or success
  type: str
  sample: test@comment
"""

import typing as t

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.crypto.plugins.module_utils._openssh.backends.keypair_backend import (
    select_backend,
)


def main() -> t.NoReturn:
    module = AnsibleModule(
        argument_spec={
            "state": {
                "type": "str",
                "default": "present",
                "choices": ["present", "absent"],
            },
            "size": {"type": "int"},
            "type": {
                "type": "str",
                "default": "rsa",
                "choices": ["rsa", "dsa", "rsa1", "ecdsa", "ed25519"],
            },
            "force": {"type": "bool", "default": False},
            "path": {"type": "path", "required": True},
            "comment": {"type": "str"},
            "regenerate": {
                "type": "str",
                "default": "partial_idempotence",
                "choices": [
                    "never",
                    "fail",
                    "partial_idempotence",
                    "full_idempotence",
                    "always",
                ],
            },
            "passphrase": {"type": "str", "no_log": True},
            "private_key_format": {
                "type": "str",
                "default": "auto",
                "no_log": False,
                "choices": ["auto", "pkcs1", "pkcs8", "ssh"],
            },
            "backend": {
                "type": "str",
                "default": "auto",
                "choices": ["auto", "cryptography", "opensshbin"],
            },
        },
        supports_check_mode=True,
        add_file_common_args=True,
    )

    keypair = select_backend(module=module, backend=module.params["backend"])

    keypair.execute()


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2015, Steve Gargan <steve.gargan@gmail.com>
# Copyright (c) 2018 Genome Research Ltd.
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: consul_kv
short_description: Manipulate entries in the key/value store of a Consul cluster
description:
  - Allows the retrieval, addition, modification and deletion of key/value entries in a Consul cluster using the agent. The
    entire contents of the record, including the indices, flags and session are returned as RV(ignore:value).
  - If the O(key) represents a prefix then note that when a value is removed, the existing value if any is returned as part
    of the results.
  - See http://www.consul.io/docs/agent/http.html#kv for more details.
requirements:
  - py-consul
  - requests
author:
  - Steve Gargan (@sgargan)
  - Colin Nolan (@colin-nolan)
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  state:
    description:
      - The action to take with the supplied key and value. If the state is V(present) and O(value) is set, the key contents
        is set to the value supplied and RV(ignore:changed) is set to V(true) only if the value was different to the current contents.
        If the state is V(present) and O(value) is not set, the existing value associated to the key is returned. The state
        V(absent) is used to remove the key/value pair, again RV(ignore:changed) is set to V(true) only if the key actually existed
        prior to the removal. An attempt can be made to obtain or free the lock associated with a key/value pair with the
        states V(acquire) or V(release) respectively. A valid session must be supplied to make the attempt RV(ignore:changed) is V(true)
        if the attempt is successful, V(false) otherwise.
    type: str
    choices: [absent, acquire, present, release]
    default: present
  key:
    description:
      - The key at which the value should be stored.
    type: str
    required: true
  value:
    description:
      - The value should be associated with the given key, required if O(state) is V(present).
    type: str
  recurse:
    description:
      - If the key represents a prefix, each entry with the prefix can be retrieved by setting this to V(true).
    type: bool
  retrieve:
    description:
      - If the O(state) is V(present) and O(value) is set, perform a read after setting the value and return this value.
    default: true
    type: bool
  session:
    description:
      - The session that should be used to acquire or release a lock associated with a key/value pair.
    type: str
  token:
    description:
      - The token key identifying an ACL rule set that controls access to the key value pair.
    type: str
  cas:
    description:
      - Used when acquiring a lock with a session. If the O(cas) is V(0), then Consul only puts the key if it does not already
        exist. If the O(cas) value is non-zero, then the key is only set if the index matches the ModifyIndex of that key.
    type: str
  flags:
    description:
      - Opaque positive integer value that can be passed when setting a value.
    type: str
  host:
    description:
      - Host of the Consul agent.
    type: str
    default: localhost
  port:
    description:
      - The port on which the Consul agent is running.
    type: int
    default: 8500
  scheme:
    description:
      - The protocol scheme on which the Consul agent is running.
    type: str
    default: http
  validate_certs:
    description:
      - Whether to verify the TLS certificate of the Consul agent.
      - Instead of setting this to V(false), please consider using O(ca_path) instead.
    type: bool
    default: true
  ca_path:
    description:
      - The CA bundle to use for HTTPS connections.
    type: str
    version_added: "12.6.0"
  datacenter:
    description:
      - The name of the datacenter to query. If unspecified, the query defaults to the datacenter of the Consul agent on O(host).
    type: str
    version_added: 10.0.0
"""


EXAMPLES = r"""
# If the key does not exist, the value associated to the "data" property in `retrieved_key` will be `None`
# If the key value is empty string, `retrieved_key["data"]["Value"]` will be `None`
- name: Retrieve a value from the key/value store
  community.general.consul_kv:
    key: somekey
  register: retrieved_key

- name: Add or update the value associated with a key in the key/value store
  community.general.consul_kv:
    key: somekey
    value: somevalue

- name: Remove a key from the store
  community.general.consul_kv:
    key: somekey
    state: absent

- name: Add a node to an arbitrary group using Consul inventory (see consul.ini)
  community.general.consul_kv:
    key: ansible/groups/dc1/somenode
    value: top_secret

- name: Register a key/value pair with an associated session
  community.general.consul_kv:
    key: stg/node/server_birthday
    value: 20160509
    session: "{{ sessionid }}"
    state: acquire
"""

from ansible.module_utils.common.text.converters import to_text

try:
    import consul
    from requests.exceptions import ConnectionError

    python_consul_installed = True
except ImportError:
    python_consul_installed = False

from ansible.module_utils.basic import AnsibleModule

# Note: although the py-consul implementation implies that using a key with a value of `None` with `put` has a special
# meaning (https://github.com/criteo/py-consul/blob/master/consul/api/kv.py), if not set in the subsequently API call,
# the value just defaults to an empty string (https://www.consul.io/api/kv.html#create-update-key)
NOT_SET = None


def _has_value_changed(consul_client, key, target_value):
    """
    Uses the given Consul client to determine if the value associated to the given key is different to the given target
    value.
    :param consul_client: Consul connected client
    :param key: key in Consul
    :param target_value: value to be associated to the key
    :return: tuple where the first element is the value of the "X-Consul-Index" header and the second is `True` if the
    value has changed (i.e. the stored value is not the target value)
    """
    index, existing = consul_client.kv.get(key)
    if not existing:
        return index, True
    try:
        changed = to_text(existing["Value"], errors="surrogate_or_strict") != target_value
        return index, changed
    except UnicodeError:
        # Existing value was not decodable but all values we set are valid utf-8
        return index, True


def execute(module):
    state = module.params.get("state")

    if state == "acquire" or state == "release":
        lock(module, state)
    elif state == "present":
        if module.params.get("value") is NOT_SET:
            get_value(module)
        else:
            set_value(module)
    elif state == "absent":
        remove_value(module)
    else:
        module.exit_json(msg=f"Unsupported state: {state}")


def lock(module, state):
    consul_api = get_consul_api(module)

    session = module.params.get("session")
    key = module.params.get("key")
    value = module.params.get("value")

    if not session:
        module.fail(msg=f"{state} of lock for {key} requested but no session supplied")

    index, changed = _has_value_changed(consul_api, key, value)

    if changed and not module.check_mode:
        if state == "acquire":
            changed = consul_api.kv.put(
                key, value, cas=module.params.get("cas"), acquire=session, flags=module.params.get("flags")
            )
        else:
            changed = consul_api.kv.put(
                key, value, cas=module.params.get("cas"), release=session, flags=module.params.get("flags")
            )

    module.exit_json(changed=changed, index=index, key=key)


def get_value(module):
    consul_api = get_consul_api(module)
    key = module.params.get("key")

    index, existing_value = consul_api.kv.get(key, recurse=module.params.get("recurse"))

    module.exit_json(changed=False, index=index, data=existing_value)


def set_value(module):
    consul_api = get_consul_api(module)

    key = module.params.get("key")
    value = module.params.get("value")

    if value is NOT_SET:
        raise AssertionError(f'Cannot set value of "{key}" to `NOT_SET`')

    index, changed = _has_value_changed(consul_api, key, value)

    if changed and not module.check_mode:
        changed = consul_api.kv.put(key, value, cas=module.params.get("cas"), flags=module.params.get("flags"))

    stored = None
    if module.params.get("retrieve"):
        index, stored = consul_api.kv.get(key)

    module.exit_json(changed=changed, index=index, key=key, data=stored)


def remove_value(module):
    """remove the value associated with the given key. if the recurse parameter
    is set then any key prefixed with the given key will be removed."""
    consul_api = get_consul_api(module)

    key = module.params.get("key")

    index, existing = consul_api.kv.get(key, recurse=module.params.get("recurse"))

    changed = existing is not None
    if changed and not module.check_mode:
        consul_api.kv.delete(key, module.params.get("recurse"))

    module.exit_json(changed=changed, index=index, key=key, data=existing)


def get_consul_api(module):
    ca_path = module.params.get("ca_path")
    validate_certs = module.params.get("validate_certs")
    verify = (ca_path or validate_certs) if validate_certs else False
    return consul.Consul(
        host=module.params.get("host"),
        port=module.params.get("port"),
        scheme=module.params.get("scheme"),
        verify=verify,
        token=module.params.get("token"),
        dc=module.params.get("datacenter"),
    )


def test_dependencies(module):
    if not python_consul_installed:
        module.fail_json(
            msg="python-consul required for this module. "
            "see https://python-consul.readthedocs.io/en/latest/#installation"
        )


def main():
    module = AnsibleModule(
        argument_spec=dict(
            cas=dict(type="str"),
            datacenter=dict(type="str"),
            flags=dict(type="str"),
            key=dict(type="str", required=True, no_log=False),
            host=dict(type="str", default="localhost"),
            scheme=dict(type="str", default="http"),
            validate_certs=dict(type="bool", default=True),
            ca_path=dict(type="str"),
            port=dict(type="int", default=8500),
            recurse=dict(type="bool"),
            retrieve=dict(type="bool", default=True),
            state=dict(type="str", default="present", choices=["absent", "acquire", "present", "release"]),
            token=dict(type="str", no_log=True),
            value=dict(type="str", default=NOT_SET),
            session=dict(type="str"),
        ),
        supports_check_mode=True,
    )

    test_dependencies(module)

    try:
        execute(module)
    except ConnectionError as e:
        module.fail_json(
            msg=f"Could not connect to consul agent at {module.params.get('host')}:{module.params.get('port')}, error was {e}"
        )
    except Exception as e:
        module.fail_json(msg=str(e))


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2017, Dag Wieers (@dagwieers) <dag@wieers.com>
# Copyright: (c) 2020, Cindy Zhao (@cizhao) <cizhao@cisco.com>
# Copyright: (c) 2023, Samita Bhattacharjee (@samitab) <samitab@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_rest
short_description: Direct access to the Cisco APIC REST API
description:
- Enables the management of the Cisco ACI fabric through direct access to the Cisco APIC REST API.
- Thanks to the idempotent nature of the APIC, this module is idempotent and reports changes.
requirements:
- lxml (when using XML payload)
- xmljson >= 0.1.8 (when using XML payload)
- python 2.7+ (when using xmljson)
options:
  method:
    description:
    - The HTTP method of the request.
    - Using C(delete) is typically used for deleting objects.
    - Using C(get) is typically used for querying objects.
    - Using C(post) is typically used for modifying objects.
    type: str
    choices: [ delete, get, post ]
    default: get
    aliases: [ action ]
  path:
    description:
    - URI being used to execute API calls.
    - Must end in C(.xml) or C(.json).
    type: str
    required: true
    aliases: [ uri ]
  content:
    description:
    - When used instead of C(src), sets the payload of the API request directly.
    - This may be convenient to template simple requests.
    - For anything complex use the C(template) lookup plugin (see examples)
      or the C(template) module with parameter C(src).
    type: raw
  src:
    description:
    - Name of the absolute path of the filename that includes the body
      of the HTTP request being sent to the ACI fabric.
    - If you require a templated payload, use the C(content) parameter
      together with the C(template) lookup plugin, or use C(template).
    type: path
    aliases: [ config_file ]
  rsp_subtree_preserve:
    description:
    - Preserve the response for the provided path.
    type: bool
    default: false
  page_size:
    description:
    - The number of items to return in a single page.
    type: int
  page:
    description:
    - The page number to return.
    type: int
  normalize_payload_values:
    description:
    - If this parameter is not specified in the task, the value of environment variable ACI_NORMALIZE_PAYLOAD_VALUES will be used instead.
    - This parameter enforces the conversion of integer and float values to strings in Ansible Core v2.19.0 and later, as well as Jinja2 v3.1.6 and later.
    - To disable this conversion, set O(normalize_payload_values=false) or unset the ACI_NORMALIZE_PAYLOAD_VALUES environment variable.
    type: bool
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation

notes:
- Certain payloads are known not to be idempotent, so be careful when constructing payloads,
  e.g. using C(status="created") will cause idempotency issues, use C(status="modified") instead.
  More information in :ref:`the ACI documentation <aci_guide_known_issues>`.
- Certain payloads (and used paths) are known to report no changes happened when changes did happen.
  This is a known APIC problem and has been reported to the vendor. A workaround for this issue exists.
  More information in :ref:`the ACI documentation <aci_guide_known_issues>`.
- XML payloads require the C(lxml) and C(xmljson) python libraries. For JSON payloads nothing special is needed.
- If you do not have any attributes, it may be necessary to add the "attributes" key with an empty dictionnary "{}" for value
  as the APIC does expect the entry to precede any children.
- Annotation set directly in c(src) or C(content) will take precedent over the C(annotation) parameter.
seealso:
- module: cisco.aci.aci_tenant
- name: Cisco APIC REST API Configuration Guide
  description: More information about the APIC REST API.
  link: http://www.cisco.com/c/en/us/td/docs/switches/datacenter/aci/apic/sw/2-x/rest_cfg/2_1_x/b_Cisco_APIC_REST_API_Configuration_Guide.html
author:
- Dag Wieers (@dagwieers)
- Cindy Zhao (@cizhao)
- Samita Bhattacharjee (@samitab)
"""

EXAMPLES = r"""
- name: Add a tenant using certificate authentication
  cisco.aci.aci_rest:
    host: apic
    username: admin
    private_key: pki/admin.key
    method: post
    path: /api/mo/uni.xml
    src: /home/cisco/ansible/aci/configs/aci_config.xml
  delegate_to: localhost

- name: Add a tenant from a templated payload file from templates/
  cisco.aci.aci_rest:
    host: apic
    username: admin
    private_key: pki/admin.key
    method: post
    path: /api/mo/uni.xml
    content: "{{ lookup('template', 'aci/tenant.xml.j2') }}"
  delegate_to: localhost

- name: Add a tenant using inline YAML
  cisco.aci.aci_rest:
    host: apic
    username: admin
    private_key: pki/admin.key
    validate_certs: false
    path: /api/mo/uni.json
    method: post
    content:
      fvTenant:
        attributes:
          name: Sales
          descr: Sales department
  delegate_to: localhost

- name: Add a tenant using a JSON string
  cisco.aci.aci_rest:
    host: apic
    username: admin
    private_key: pki/admin.key
    validate_certs: false
    path: /api/mo/uni.json
    method: post
    content:
      {
        "fvTenant": {
          "attributes": {
            "name": "Sales",
            "descr": "Sales department"
          }
        }
      }
  delegate_to: localhost

- name: Add a tenant using an XML string
  cisco.aci.aci_rest:
    host: apic
    username: admin
    private_key: pki/{{ aci_username }}.key
    validate_certs: false
    path: /api/mo/uni.xml
    method: post
    content: '<fvTenant name="Sales" descr="Sales departement"/>'
  delegate_to: localhost

- name: Get tenants using password authentication
  cisco.aci.aci_rest:
    host: apic
    username: admin
    password: SomeSecretPassword
    method: get
    path: /api/node/class/fvTenant.json
  delegate_to: localhost
  register: query_result

- name: Get first 5 tenants using password authentication and pagination
  cisco.aci.aci_rest:
    host: apic
    username: admin
    password: SomeSecretPassword
    method: get
    page_size: 5
    path: /api/node/class/fvTenant.json
  delegate_to: localhost
  register: query_result

- name: Configure contracts
  cisco.aci.aci_rest:
    host: apic
    username: admin
    private_key: pki/admin.key
    method: post
    path: /api/mo/uni.xml
    src: /home/cisco/ansible/aci/configs/contract_config.xml
  delegate_to: localhost

- name: Register leaves and spines
  cisco.aci.aci_rest:
    host: apic
    username: admin
    private_key: pki/admin.key
    validate_certs: false
    method: post
    path: /api/mo/uni/controller/nodeidentpol.xml
    content:
      <fabricNodeIdentPol>
        <fabricNodeIdentP name="{{ item.name }}" nodeId="{{ item.nodeid }}" status="{{ item.status }}" serial="{{ item.serial }}"/>
      </fabricNodeIdentPol>
  with_items:
    - '{{ apic_leavesspines }}'
  delegate_to: localhost

- name: Wait for all controllers to become ready
  cisco.aci.aci_rest:
    host: apic
    username: admin
    private_key: pki/admin.key
    validate_certs: false
    path: /api/node/class/topSystem.json?query-target-filter=eq(topSystem.role,"controller")
  register: apics
  until: "'totalCount' in apics and apics.totalCount|int >= groups['apic']|count"
  retries: 120
  delay: 30
  delegate_to: localhost
  run_once: true
"""

RETURN = r"""
error_code:
  description: The REST ACI return code, useful for troubleshooting on failure
  returned: always
  type: int
  sample: 122
error_text:
  description: The REST ACI descriptive text, useful for troubleshooting on failure
  returned: always
  type: str
  sample: unknown managed object class foo
imdata:
  description: Converted output returned by the APIC REST (register this for post-processing)
  returned: always
  type: str
  sample: [{"error": {"attributes": {"code": "122", "text": "unknown managed object class foo"}}}]
payload:
  description: The (templated) payload send to the APIC REST API (xml or json)
  returned: always
  type: str
  sample: '<foo bar="boo"/>'
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
response:
  description: HTTP response string
  returned: always
  type: str
  sample: 'HTTP Error 400: Bad Request'
status:
  description: HTTP status code
  returned: always
  type: int
  sample: 400
totalCount:
  description: Number of items in the imdata array
  returned: always
  type: str
  sample: '0'
url:
  description: URL used for APIC REST call
  returned: success
  type: str
  sample: https://1.2.3.4/api/mo/uni/tn-[Dag].json?rsp-subtree=modified
"""

import json
import os
import re

try:
    from ansible.module_utils.six.moves.urllib.parse import parse_qsl, urlencode, urlparse, urlunparse

    HAS_URLPARSE = True
except Exception:
    HAS_URLPARSE = False

# Optional, only used for XML payload
try:
    from lxml import etree  # noqa

    HAS_LXML_ETREE = True
except ImportError:
    HAS_LXML_ETREE = False

# Optional, only used for XML payload
try:
    from xmljson import cobra  # noqa

    HAS_XMLJSON_COBRA = True
except ImportError:
    HAS_XMLJSON_COBRA = False

# Optional, only used for YAML validation
try:
    import yaml

    HAS_YAML = True
except Exception:
    HAS_YAML = False

from ansible.module_utils.basic import AnsibleModule, env_fallback
from ansible_collections.cisco.aci.plugins.module_utils.aci import (
    ACIModule,
    aci_argument_spec,
    aci_annotation_spec,
    convert_numbers_and_none_values_to_string,
)
from ansible.module_utils._text import to_text
from ansible_collections.cisco.aci.plugins.module_utils.annotation_unsupported import (
    ANNOTATION_UNSUPPORTED,
)


def update_qsl(url, params):
    """Add or update a URL query string"""

    if HAS_URLPARSE:
        url_parts = list(urlparse(url))
        query = dict(parse_qsl(url_parts[4]))
        query.update(params)
        url_parts[4] = urlencode(query)
        return urlunparse(url_parts)
    elif "?" in url:
        return url + "&" + "&".join(["%s=%s" % (k, v) for k, v in params.items()])
    else:
        return url + "?" + "&".join(["%s=%s" % (k, v) for k, v in params.items()])


def add_annotation(annotation, payload):
    """Add annotation to payload only if it has not already been added"""
    if annotation and isinstance(payload, dict):
        for key, val in payload.items():
            if key in ANNOTATION_UNSUPPORTED:
                continue
            if isinstance(val, dict):
                att = val.get("attributes", {})
                if "annotation" not in att.keys():
                    att["annotation"] = annotation
                # Recursively add annotation to children
                children = val.get("children", None)
                if children:
                    for child in children:
                        add_annotation(annotation, child)


def add_annotation_xml(annotation, tree):
    """Add annotation to payload xml only if it has not already been added"""
    if annotation:
        for element in tree.iter():
            if element.tag in ANNOTATION_UNSUPPORTED:
                continue
            ann = element.get("annotation")
            if ann is None:
                element.set("annotation", annotation)


class ACIRESTModule(ACIModule):
    def changed(self, d):
        """Check ACI response for changes"""

        if isinstance(d, dict):
            for k, v in d.items():
                if k == "status" and v in ("created", "modified", "deleted"):
                    return True
                elif self.changed(v) is True:
                    return True
        elif isinstance(d, list):
            for i in d:
                if self.changed(i) is True:
                    return True

        return False

    def response_type(self, rawoutput, rest_type="xml"):
        """Handle APIC response output"""

        if rest_type == "json":
            self.response_json(rawoutput)
        else:
            self.response_xml(rawoutput)

        # Use APICs built-in idempotency
        if HAS_URLPARSE:
            self.result["changed"] = self.changed(self.imdata)


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(
        path=dict(type="str", required=True, aliases=["uri"]),
        method=dict(type="str", default="get", choices=["delete", "get", "post"], aliases=["action"]),
        src=dict(type="path", aliases=["config_file"]),
        content=dict(type="raw"),
        rsp_subtree_preserve=dict(type="bool", default=False),
        page_size=dict(type="int"),
        page=dict(type="int"),
        # To support Ansible Core 2.19.0 and later, Jinja2 3.1.6 and later versions.
        normalize_payload_values=dict(
            type="bool",
            fallback=(env_fallback, ["ACI_NORMALIZE_PAYLOAD_VALUES"]),
        ),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        mutually_exclusive=[["content", "src"]],
    )

    content = module.params.get("content")
    path = module.params.get("path")
    src = module.params.get("src")
    rsp_subtree_preserve = module.params.get("rsp_subtree_preserve")
    annotation = module.params.get("annotation")
    page_size = module.params.get("page_size")
    page = module.params.get("page")
    normalize_payload_values = module.params.get("normalize_payload_values")
    if module.params.get("method") != "get" and page_size:
        module.fail_json(msg="Pagination parameters (page and page_size) are only valid for GET method")

    # Report missing file
    file_exists = False
    if src:
        if os.path.isfile(src):
            file_exists = True
        else:
            module.fail_json(msg="Cannot find/access src '{0}'".format(src))

    # Find request type
    if path.find(".xml") != -1:
        rest_type = "xml"
        if not HAS_LXML_ETREE:
            module.fail_json(msg="The lxml python library is missing, or lacks etree support.")
        if not HAS_XMLJSON_COBRA:
            module.fail_json(msg="The xmljson python library is missing, or lacks cobra support.")
    elif path.find(".json") != -1:
        rest_type = "json"
    else:
        module.fail_json(msg="Failed to find REST API payload type (neither .xml nor .json).")

    aci = ACIRESTModule(module)
    aci.result["status"] = -1  # Ensure we always return a status

    # We include the payload as it may be templated
    payload = content
    if file_exists:
        with open(src, "r") as config_object:
            # TODO: Would be nice to template this, requires action-plugin
            payload = config_object.read()

    # Validate payload
    if rest_type == "json" and payload:
        if isinstance(payload, str) and HAS_YAML:
            try:
                payload = yaml.safe_load(payload)
            except Exception as e:
                module.fail_json(msg="Failed to parse provided JSON/YAML payload: {0}".format(to_text(e)), exception=to_text(e), payload=payload)
        add_annotation(annotation, payload)
        payload = json.dumps(convert_numbers_and_none_values_to_string(payload) if normalize_payload_values else payload)

    elif rest_type == "xml" and HAS_LXML_ETREE:
        if payload and isinstance(payload, dict) and HAS_XMLJSON_COBRA:
            # Validate inline YAML/JSON
            add_annotation(annotation, payload)
            payload = etree.tostring(cobra.etree(payload)[0], encoding="unicode")
        elif payload and isinstance(payload, str):
            try:
                # Validate XML string
                payload = etree.fromstring(payload)
                add_annotation_xml(annotation, payload)
                payload = etree.tostring(payload, encoding="unicode")
            except Exception as e:
                module.fail_json(msg="Failed to parse provided XML payload: {0}".format(to_text(e)), payload=payload)

    # Perform actual request using auth cookie (Same as aci.request(), but also supports XML)
    # NOTE By setting aci.path we ensure that Ansible displays accurate URL info when the plugin and the aci_rest module are used.
    aci.path = path.lstrip("/")
    aci.url = "{0}/{1}".format(aci.base_url, aci.path)

    if aci.params.get("method") == "get" and page_size:
        aci.path = update_qsl(aci.path, {"page": page, "page-size": page_size})
        aci.url = update_qsl(aci.url, {"page": page, "page-size": page_size})
    if aci.params.get("method") != "get" and not rsp_subtree_preserve:
        aci.path = "{0}?rsp-subtree=modified".format(aci.path)
        aci.url = update_qsl(aci.url, {"rsp-subtree": "modified"})

    method = aci.params.get("method").upper()
    # Perform request
    if not aci.module.check_mode:
        resp, info = aci.api_call(method, aci.url, data=payload, return_response=True)
        # Report failure
        if info.get("status") != 200:
            try:
                # APIC error
                aci.response_type(info["body"], rest_type)
                aci.fail_json(msg="APIC Error {code}: {text}".format_map(aci.error))
            except KeyError:
                # Connection error
                aci.fail_json(msg="Connection failed for {url}. {msg}".format_map(info))

        try:
            aci.response_type(resp.read(), rest_type)
        except AttributeError:
            aci.response_type(info.get("body"), rest_type)

        aci.result["status"] = aci.status
        aci.result["imdata"] = aci.imdata
        aci.result["totalCount"] = aci.totalCount

    else:
        # NOTE A case when aci_rest is used with check mode and the apic host is used directly from the inventory
        if aci.connection is not None and aci.params.get("host") is None:
            aci.url = urlunparse(urlparse(aci.url)._replace(netloc=re.sub(r"[[\]]", "", aci.connection.get_option("host")).split(",")[0]))
        aci.method = method
        # Set changed to true so check_mode changed result is behaving similar to non aci_rest modules
        aci.result["changed"] = True

    # Only set proposed if we have a payload and thus also only allow output_path if we have a payload
    # DELETE and GET do not have a payload
    if payload and method == "POST":
        if rest_type == "json":
            payload = json.loads(payload)

        aci.result["proposed"] = payload

        output_path = aci.params.get("output_path")
        if output_path is not None:
            with open(output_path, "a") as output_file:
                if rest_type == "json":
                    json.dump([payload], output_file)
                else:
                    output_file.write(str(payload))

    # Report success
    aci.exit_json(**aci.result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2025, Akini Ross (@akinross) <akinross@cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: ndo_l3out_node_static_route
version_added: "2.11.0"
short_description: Manage L3Out Node Static Routes on Cisco Nexus Dashboard Orchestrator (NDO).
description:
- Manage L3Out Node Static Routes on Cisco Nexus Dashboard Orchestrator (NDO).
- This module is only supported on ND v3.1 (NDO v4.3) and later.
author:
- Akini Ross (@akinross)
options:
  template:
    description:
    - The name of the template.
    - The template must be an L3Out template.
    - This parameter or O(template_id) is required.
    type: str
    aliases: [ l3out_template ]
  template_id:
    description:
    - The ID of the L3Out template.
    - This parameter or O(template) is required.
    type: str
    aliases: [ l3out_template_id ]
  l3out:
    description:
    - The name of the L3Out.
    - This parameter or O(l3out_uuid) is required.
    type: str
    aliases: [ l3out_name ]
  l3out_uuid:
    description:
    - The UUID of the L3Out.
    - This parameter or O(l3out) is required.
    type: str
  pod_id:
    description:
    - The pod ID of the node (border leaf switch).
    type: str
    required: true
    aliases: [ pod ]
  node_id:
    description:
    - The ID of the node.
    type: str
    required: true
    aliases: [ node, border_leaf ]
  prefix:
    description:
    - The prefix of the static route.
    type: str
  description:
    description:
    - The description of the static route.
    - Providing an empty string O(description="") will remove the description from the static route.
    type: str
  administrative_distance:
    description:
    - The administrative distance of the static route.
    - The value must be in the range 1 - 255.
    - Defaults to O(administrative_distance=1) when unset during creation.
    type: int
    aliases: [ admin_distance ]
  bfd_tracking:
    description:
    - Enable BFD tracking for the static route.
    - Defaults to O(bfd_tracking=false) when unset during creation.
    type: bool
  next_hop_null:
    description:
    - Create a static route to Null0 interface.
    - This is used to drop traffic for the specified prefix.
    - Defaults to O(next_hop_null=false) when unset during creation.
    type: bool
  track_policy_uuid:
    description:
    - The UUID of the track policy to be used.
    - Providing an empty string O(track_policy="") will remove the track policy from the static route.
    type: str
    aliases: [ ipsla_track_list_uuid ]
  track_policy:
    description:
    - The track policy to be used.
    - Providing an empty dictionary O(track_policy={}) will remove the track policy from the static route.
    type: dict
    aliases: [ ipsla_track_list ]
    suboptions:
      name:
        description:
        - The name of the track policy.
        type: str
      template:
        description:
        - The name of the template that contains the track policy.
        - This parameter or O(track_policy.template_id) is required.
        type: str
      template_id:
        description:
        - The ID of the template that contains the track policy.
        - This parameter or O(track_policy.template) is required.
        type: str
  state:
    description:
    - Determines the desired state of the resource.
    - Use C(absent) to remove the resource.
    - Use C(query) to list the resource.
    - Use C(present) to create or update the resource.
    type: str
    choices: [ absent, query, present ]
    default: query
notes:
- The O(template) or O(template_id) must exist before using this module in your playbook.
  The M(cisco.mso.ndo_template) module can be used for this.
- The O(l3out) or O(l3out_uuid) must exist before using this module in your playbook.
  The M(cisco.mso.ndo_l3out_template) module can be used for this.
- The O(track_policy) or O(track_policy_uuid) must exist before using this module in your playbook.
  The M(cisco.mso.ndo_ipsla_track_list) module can be used for this.
seealso:
- module: cisco.mso.ndo_template
- module: cisco.mso.ndo_l3out_template
- module: cisco.mso.ndo_ipsla_track_list
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Create a L3Out Node Static Route
  cisco.mso.ndo_l3out_node_static_route:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: l3out_template
    l3out: l3out_name
    pod_id: 1
    node_id: 101
    prefix: 10.0.0.0/24
    state: present

- name: Update an existing L3Out Node Static Route
  cisco.mso.ndo_l3out_node_static_route:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template_id: "{{ l3out_template.current.templateId }}"
    l3out_uuid: "{{ l3out.current.uuid }}"
    pod_id: 1
    node_id: 101
    prefix: 10.0.0.0/24
    description: "Static route for 10.0.0.0/24"
    administrative_distance: 10
    bfd_tracking: true
    next_hop_null: true
    track_policy:
      name: "track_policy_name"
      template: "track_policy_template"
    state: present

- name: Query an existing L3Out Node Static Route
  cisco.mso.ndo_l3out_node_static_route:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: l3out_template
    l3out: l3out_name
    pod_id: 1
    node_id: 101
    prefix: 10.0.0.0/24
    state: query
  register: query_with_name

- name: Query all existing L3Out Node Static Routes in a L3Out Node
  cisco.mso.ndo_l3out_node_static_route:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: l3out_template
    l3out: l3out_name
    pod_id: 1
    node_id: 101
    state: query

- name: Delete an existing L3Out Node Static Route
  cisco.mso.ndo_l3out_node_static_route:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: l3out_template
    l3out: l3out_name
    pod_id: 1
    node_id: 101
    prefix: 10.0.0.0/24
    state: absent
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec
from ansible_collections.cisco.mso.plugins.module_utils.templates import MSOTemplates
from ansible_collections.cisco.mso.plugins.module_utils.utils import append_update_ops_data, check_if_all_elements_are_none
import copy


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        template=dict(type="str", aliases=["l3out_template"]),
        template_id=dict(type="str", aliases=["l3out_template_id"]),
        l3out=dict(type="str", aliases=["l3out_name"]),
        l3out_uuid=dict(type="str"),
        pod_id=dict(type="str", aliases=["pod"], required=True),
        node_id=dict(type="str", aliases=["node", "border_leaf"], required=True),
        prefix=dict(type="str"),
        description=dict(type="str"),
        administrative_distance=dict(type="int", aliases=["admin_distance"]),
        bfd_tracking=dict(type="bool"),
        next_hop_null=dict(type="bool"),
        track_policy_uuid=dict(type="str", aliases=["ipsla_track_list_uuid"]),
        track_policy=dict(
            type="dict",
            aliases=["ipsla_track_list"],
            options=dict(
                name=dict(type="str"),
                template=dict(type="str"),
                template_id=dict(type="str"),
            ),
            required_by={
                "template": "name",
                "template_id": "name",
            },
            mutually_exclusive=[
                ["template", "template_id"],
            ],
        ),
        state=dict(type="str", default="query", choices=["absent", "query", "present"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "present", ["prefix"]],
            ["state", "absent", ["prefix"]],
        ],
        required_one_of=[
            ["template", "template_id"],
            ["l3out", "l3out_uuid"],
        ],
        mutually_exclusive=[
            ["template", "template_id"],
            ["l3out", "l3out_uuid"],
            ["track_policy_uuid", "track_policy"],
        ],
    )

    mso = MSOModule(module)
    mso_templates = MSOTemplates(mso)

    template_name = mso.params.get("template")
    template_id = mso.params.get("template_id")
    l3out = mso.params.get("l3out")
    l3out_uuid = mso.params.get("l3out_uuid")
    pod_id = mso.params.get("pod_id")
    node_id = mso.params.get("node_id")
    prefix = mso.params.get("prefix")
    description = mso.params.get("description")
    administrative_distance = mso.params.get("administrative_distance")
    bfd_tracking = mso.params.get("bfd_tracking")
    next_hop_null = mso.params.get("next_hop_null")
    track_policy_uuid = mso.params.get("track_policy_uuid")
    track_policy = mso.params.get("track_policy")
    if track_policy is not None and check_if_all_elements_are_none(track_policy.values()):
        track_policy = {}
    if track_policy and not (track_policy.get("name") and (track_policy.get("template") or track_policy.get("template_id"))):
        mso.fail_json(msg="track_policy.name and one of the following are required: track_policy.template, track_policy.template_id")

    state = mso.params.get("state")

    mso_template = mso_templates.get_template("l3out", template_name, template_id)
    mso_template.validate_template("l3out")
    l3out_object = mso_template.get_l3out_object(l3out_uuid, l3out, True)
    node_object = mso_template.get_l3out_node(l3out_object.details, pod_id, node_id, True)

    if track_policy or track_policy_uuid:
        if track_policy_uuid:
            track_policy_match = mso_template.get_template_object_by_uuid("ipslaTrackList", track_policy_uuid, True)
        else:
            track_policy_mso_template = mso_templates.get_template(
                "tenant",
                track_policy.get("template"),
                track_policy.get("template_id"),
                fail_module=True,
            )
            track_policy_match = track_policy_mso_template.get_ipsla_track_list(
                uuid=track_policy_uuid,
                name=track_policy.get("name"),
                fail_module=True,
            )
            track_policy_uuid = track_policy_match.details.get("uuid")

    match = mso_template.get_l3out_node_static_route(node_object.details, prefix)
    if prefix and match:
        set_node_static_route_details(mso_template, match.details)
        mso.existing = mso.previous = copy.deepcopy(match.details)  # Query a specific object
    elif match:
        mso.existing = [set_node_static_route_details(mso_template, obj) for obj in match]

    l3out_node_static_route_path = "/l3outTemplate/l3outs/{0}/nodes/{1}/staticRoutes/{2}".format(
        l3out_object.index, node_object.index, match.index if match else "-"
    )

    ops = []

    if state == "present":
        mso_values = {
            "prefix": prefix,
            "description": description if description else None,
            "fallbackPref": administrative_distance,
            "enableBFDTracking": bfd_tracking,
            "trackPolicyRef": track_policy_uuid if track_policy_uuid else None,
            "nullNextHop": next_hop_null,
        }

        if match:
            remove_data = []
            unwanted = []
            if description == "" and match.details.get("description"):
                remove_data.append("description")
                unwanted.append("description")
            if track_policy == {} or track_policy_uuid == "":
                remove_data.extend(["trackPolicyRef"])
                unwanted.extend(["trackPolicyRef", "trackPolicyName", "trackPolicyTemplateName", "trackPolicyTemplateId"])
            append_update_ops_data(ops, match.details, l3out_node_static_route_path, mso_values, remove_data)
            mso.sanitize(mso_values, collate=True, unwanted=unwanted)
        else:
            mso.sanitize(mso_values)
            ops.append(dict(op="add", path=l3out_node_static_route_path, value=mso.sent))

        set_node_static_route_details(mso_template, mso.proposed)

    elif state == "absent":
        if match:
            ops.append(dict(op="remove", path=l3out_node_static_route_path))

    if not mso.module.check_mode and ops:
        response = mso.request(mso_template.template_path, method="PATCH", data=ops)
        l3out_object = mso_template.get_l3out_object(l3out_uuid, l3out, True, search_object=response)
        node_object = mso_template.get_l3out_node(l3out_object.details, pod_id, node_id, True)
        match = mso_template.get_l3out_node_static_route(node_object.details, prefix)
        if match:
            set_node_static_route_details(mso_template, match.details)
            mso.existing = match.details  # When the state is present
        else:
            mso.existing = {}  # When the state is absent
    elif mso.module.check_mode and state != "query":  # When the state is present/absent with check mode
        set_node_static_route_details(mso_template, mso.proposed)
        mso.existing = mso.proposed if state == "present" else {}
    mso.exit_json()


def set_node_static_route_details(mso_template, node_static_route):
    if node_static_route:
        reference_details = None
        if node_static_route.get("trackPolicyRef"):
            reference_details = {
                "track_list_reference": {
                    "name": "trackPolicyName",
                    "reference": "trackPolicyRef",
                    "type": "ipslaTrackList",
                    "template": "trackPolicyTemplateName",
                    "templateId": "trackPolicyTemplateId",
                }
            }
        mso_template.update_config_with_template_and_references(
            node_static_route,
            reference_details,
            True,
        )

    return node_static_route


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2024, Sabari Jaganathan (@sajagana) <sajagana@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}


DOCUMENTATION = r"""
---
module: mso_schema_site_contract_service_graph_listener
short_description: Manage the listener for Azure site contract service graph in schema sites
description:
- Manage the listener for Azure site contract service graph in schema sites on Cisco ACI Multi-Site.
- This module is only compatible with NDO versions 3.7 and 4.2+. NDO versions 4.0 and 4.1 are not supported.
author:
- Sabari Jaganathan (@sajagana)
options:
  tenant:
    description:
    - The name of the tenant.
    type: str
  device:
    description:
    - The name of the device.
    type: str
    aliases: [ device_name ]
  schema:
    description:
    - The name of the schema.
    type: str
    required: true
  template:
    description:
    - The name of the template.
    type: str
    required: true
  contract:
    description:
    - The name of the contract.
    type: str
    required: true
  site:
    description:
    - The name of the site.
    type: str
    required: true
  service_node_index:
    description:
    - The index of the service node in the site contract service graph. The value starts from 0.
    type: int
  listener:
    description:
    - The name of the listener.
    type: str
    aliases: [ name, listener_name ]
  listener_protocol:
    description:
    - The protocol of the listener.
    type: str
    choices: [ http, https, tcp, udp, tls, inherit ]
  listener_port:
    description:
    - The port of the listener.
    type: int
  security_policy:
    description:
    - The security policy of the listener.
    type: str
    choices: [
        default,
        elb_sec_2016_18,
        elb_sec_fs_2018_06,
        elb_sec_tls_1_2_2017_01,
        elb_sec_tls_1_2_ext_2018_06,
        elb_sec_tls_1_1_2017_01,
        elb_sec_2015_05,
        elb_sec_tls_1_0_2015_04,
        app_gw_ssl_default,
        app_gw_ssl_2015_501,
        app_gw_ssl_2017_401,
        app_gw_ssl_2017_401s
      ]
  ssl_certificates:
    description:
    - The ssl certificates of the listener.
    type: list
    elements: dict
    suboptions:
      name:
        description:
        - The name of the ssl certificate.
        type: str
        required: true
      certificate_store:
        description:
        - The certificate store of the ssl certificate.
        type: str
        required: true
        choices: [ default, iam, acm ]
  frontend_ip:
    description:
    - The frontend ip of the listener. Only supported for Network load balancers.
    type: str
  rules:
    description:
    - The rules of the listener.
    type: list
    elements: dict
    suboptions:
      name:
        description:
        - The name of the rule.
        type: str
        required: true
      floating_ip:
        description:
        - The floating ip of the rule.
        type: str
      priority:
        description:
        - The priority of the rule.
        type: int
        required: true
      host:
        description:
        - The host of the rule.
        type: str
      path:
        description:
        - The path of the rule.
        type: str
      action:
        description:
        - The action of the rule.
        type: str
      action_type:
        description:
        - The action type of the rule.
        type: str
        required: true
        choices: [ fixed_response, forward, redirect, ha_port ]
      content_type:
        description:
        - The content type of the rule.
        type: str
        choices: [ text_plain, text_css, text_html, app_js, app_json ]
      port:
        description:
        - The port of the rule.
        type: int
      protocol:
        description:
        - The protocol of the rule.
        type: str
        choices: [ http, https, tcp, udp, tls, inherit ]
      provider_epg:
        description:
        - The provider epg of the rule.
        type: dict
        suboptions:
          schema:
            description:
            - The schema name of the provider epg reference.
            type: str
          template:
            description:
            - The template name of the provider epg reference.
            type: str
          anp_name:
            description:
            - The application profile name of the provider epg reference.
            type: str
            required: true
            aliases: [ anp ]
          epg_name:
            description:
            - The epg (Endpoint Group) name of the provider epg reference.
            type: str
            required: true
            aliases: [ epg ]
      url_type:
        description:
        - The url type of the rule.
        type: str
        choices: [ original, custom ]
      custom_url:
        description:
        - The custom url of the rule.
        type: str
      redirect_host_name:
        description:
        - The redirect host name of the rule.
        type: str
      redirect_path:
        description:
        - The redirect path of the rule.
        type: str
      redirect_query:
        description:
        - The redirect query of the rule.
        type: str
      response_code:
        description:
        - The response code of the rule.
        type: str
      response_body:
        description:
        - The response body of the rule.
        type: str
      redirect_protocol:
        description:
        - The redirect protocol of the rule.
        type: str
        choices: [ http, https, tcp, udp, tls, inherit ]
      redirect_port:
        description:
        - The redirect port of the rule.
        type: int
      redirect_code:
        description:
        - The redirect code of the rule.
        type: str
        choices: [ unknown, permanently_moved, found, see_other, temporary_redirect ]
      health_check:
        description:
        - The health check of the rule.
        type: dict
        suboptions:
          port:
            description:
            - The port of the health check.
            type: int
          protocol:
            description:
            - The protocol of the health check.
            type: str
            choices: [ http, https, tcp, udp, tls, inherit ]
          path:
            description:
            - The path of the health check.
            type: str
          interval:
            description:
            - The interval of the health check.
            type: int
          timeout:
            description:
            - The timeout of the health check.
            type: int
          unhealthy_threshold:
            description:
            - The unhealthy threshold of the health check.
            type: int
          use_host_from_rule:
            description:
            - The use host from rule of the health check.
            type: bool
          host:
            description:
            - The host of the health check. The host attribute will be enabled when the I(use_host_from_rule) is false.
            type: str
          success_code:
            description:
            - The success code of the health check.
            type: str
      target_ip_type:
        description:
        - The target ip type of the rule.
        type: str
        choices: [ unspecified, primary, secondary ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
seealso:
- module: cisco.mso.mso_schema_template_contract_service_graph
extends_documentation_fragment: cisco.mso.modules
"""


EXAMPLES = r"""
- name: Add a listener for Network Load-Balancer
  cisco.mso.mso_schema_site_contract_service_graph_listener:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    contract: "Contract2"
    schema: mso_schema
    template: ansible_template1
    site: mso_site
    service_node_index: 0
    listener: nlb_li_tcp
    listener_port: 80
    listener_protocol: tcp
    tenant: mso_tenant
    frontend_ip: "10.10.10.10"
    device: ans_test_nlb
    security_policy: default
    rules:
      - name: rule1
        priority: 1
        action_type: forward
        port: 80
        protocol: tcp
        health_check:
          port: 80
          protocol: tcp
          interval: 5
          unhealthy_threshold: 2
          success_code: 200-399

- name: Add a listener for Application Load-Balancer
  cisco.mso.mso_schema_site_contract_service_graph_listener:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    contract: "Contract2"
    schema: mso_schema
    template: ansible_template1
    site: mso_site
    service_node_index: 1
    listener: aplb_li_https
    tenant: mso_tenant
    device: ans_test_aplb
    listener_port: 443
    listener_protocol: https
    security_policy: default
    ssl_certificates:
      - name: ans_test_keyring
        certificate_store: default
    rules:
      - name: rule1
        priority: 1
        action_type: forward
        port: 80
        protocol: http
        provider_epg:
          anp_name: AP1
          epg_name: EPG1
        health_check:
          port: 80
          protocol: http
          path: "health_check_path"
          interval: 30
          timeout: 30
          unhealthy_threshold: 3
          use_host_from_rule: true
          success_code: "200"
        target_ip_type: unspecified

- name: Query all listeners
  cisco.mso.mso_schema_site_contract_service_graph_listener:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    contract: "Contract2"
    schema: mso_schema
    template: ansible_template1
    site: mso_site
    state: query
  register: query_all_listeners

- name: Query all listeners with name ans_li_common
  cisco.mso.mso_schema_site_contract_service_graph_listener:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    contract: "Contract2"
    schema: mso_schema
    template: ansible_template1
    site: mso_site
    listener: ans_li_common
    state: query
  register: query_all_ans_li_common

- name: Query a listener with name - aplb_li_https
  cisco.mso.mso_schema_site_contract_service_graph_listener:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    contract: "Contract2"
    schema: mso_schema
    template: ansible_template1
    site: mso_site
    service_node_index: 1
    listener: aplb_li_https
    state: query
  register: query_aplb_li_https

- name: Remove an existing listener - ans_li_common
  cisco.mso.mso_schema_site_contract_service_graph_listener:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    contract: "Contract2"
    schema: mso_schema
    template: ansible_template1
    site: mso_site
    service_node_index: 1
    listener: aplb_li_https
    state: absent
"""

RETURN = r"""
"""


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.schema import MSOSchema
from ansible_collections.cisco.mso.plugins.module_utils.mso import (
    MSOModule,
    mso_argument_spec,
    listener_ssl_certificates_spec,
    listener_rules_spec,
)
from ansible_collections.cisco.mso.plugins.module_utils.constants import (
    LISTENER_REDIRECT_CODE_MAP,
    LISTENER_CONTENT_TYPE_MAP,
    LISTENER_ACTION_TYPE_MAP,
    LISTENER_SECURITY_POLICY_MAP,
    LISTENER_PROTOCOLS,
    YES_OR_NO_TO_BOOL_STRING_MAP,
)


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        tenant=dict(type="str"),
        device=dict(type="str", aliases=["device_name"]),
        schema=dict(type="str", required=True),
        template=dict(type="str", required=True),
        contract=dict(type="str", required=True),
        site=dict(type="str", required=True),
        service_node_index=dict(type="int"),
        listener=dict(type="str", aliases=["name", "listener_name"]),
        listener_protocol=dict(type="str", choices=LISTENER_PROTOCOLS),
        listener_port=dict(type="int"),
        security_policy=dict(type="str", choices=list(LISTENER_SECURITY_POLICY_MAP)),
        ssl_certificates=dict(type="list", elements="dict", options=listener_ssl_certificates_spec()),
        frontend_ip=dict(type="str"),
        rules=dict(type="list", elements="dict", options=listener_rules_spec()),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["listener"]],
            ["state", "present", ["listener", "listener_protocol", "listener_port", "rules"]],
        ],
    )

    mso = MSOModule(module)
    site = module.params.get("site")

    # Get site id
    site_id = mso.lookup_site(site)

    if mso.site_type == "on-premise" or mso.cloud_provider_type != "azure":
        mso.fail_json(msg="The Site Contract Service Graph Listener is not supported for the site: {0}.".format(site))

    schema = module.params.get("schema")
    template = module.params.get("template")
    contract = module.params.get("contract")
    service_node_index = module.params.get("service_node_index")
    listener = module.params.get("listener")
    tenant = module.params.get("tenant")
    device = module.params.get("device")
    listener_protocol = module.params.get("listener_protocol")
    listener_port = module.params.get("listener_port")
    security_policy = LISTENER_SECURITY_POLICY_MAP.get(module.params.get("security_policy"))
    ssl_certificates = module.params.get("ssl_certificates")
    frontend_ip = module.params.get("frontend_ip")
    rules = module.params.get("rules")
    state = module.params.get("state")

    if listener_protocol == "https":
        mso.input_validation("listener_protocol", "https", ["security_policy", "ssl_certificates"], module.params, None, object_name=listener)

    mso_schema = MSOSchema(mso, schema, template)
    mso_schema.set_template(template)
    schema_id = mso.lookup_schema(schema, True)
    mso_schema.set_site(template, site)
    mso_schema.set_site_contract(contract, False)

    service_graph_ref = mso_schema.schema_objects["site_contract"].details.get("serviceGraphRelationship", {}).get("serviceGraphRef")
    service_nodes = mso_schema.schema_objects["site_contract"].details.get("serviceGraphRelationship", {}).get("serviceNodesRelationship", [])

    parent_present = False

    if service_graph_ref is None:
        mso.fail_json(msg="The site contract: {0} is not associated with a service graph.".format(contract))
    # Parent object present check
    else:
        if service_node_index is not None and service_node_index >= 0:
            if len(service_nodes) > 0 and len(service_nodes) > service_node_index:
                service_node = service_nodes[service_node_index]
                # Query all listeners under a contract service node
                # The below condition was never false if the service graph was created properly, but condition is required to avoid error
                if service_node.get("serviceNodeRef") and service_node.get("serviceNodeRef").split("/")[-1] == "node{0}".format(service_node_index + 1):
                    listeners = service_node.get("deviceConfiguration", {}).get("cloudLoadBalancer", {}).get("listeners", [])
                    if listeners:
                        parent_present = True
                        if listener:
                            for listener_data in listeners:
                                if listener_data.get("name") == listener:
                                    mso.existing = listener_data
                                    break
                        else:
                            mso.existing = listeners
                else:
                    mso.fail_json(
                        msg="The service_node_index: {0} is not matching with the service node reference: {1}.".format(
                            service_node_index, service_node.get("serviceNodeRef")
                        )
                    )
            else:
                mso.fail_json(msg="The service_node_index: {0} is out of range.".format(service_node_index))

        # Query all listeners under a contract does not require service_node_index, so the below condition is required
        elif state == "query":
            # Query all listeners under a contract
            for service_node in service_nodes:
                listeners = service_node.get("deviceConfiguration", {}).get("cloudLoadBalancer", {}).get("listeners", [])
                if listener:
                    for listener_data in listeners:
                        # Query a listener under a contract service node
                        if listener_data.get("name") == listener:
                            mso.existing = ([listener_data] + mso.existing) if mso.existing else [listener_data]
                            break
                else:
                    mso.existing = (listeners + mso.existing) if mso.existing else listeners

        else:
            mso.fail_json(msg="The service_node_index: {0} is not valid.".format(service_node_index))

    if state == "query":
        mso.exit_json()

    ops = []
    mso.previous = mso.existing

    parent_object = {}

    if state == "present":
        # Parent object creation logic begins
        if device is None and parent_present is False:
            mso.fail_json(msg="The 'device' name is required to initialize the parent object.")

        elif device is not None and parent_present is False:
            query_device_data = mso.lookup_service_node_device(site_id, tenant, device_name=device)

            if query_device_data.get("deviceVendorType") == "NATIVELB" and (
                query_device_data.get("devType") == "application" or query_device_data.get("devType") == "network"
            ):
                mso_schema.set_site_service_graph(service_graph_ref.split("/")[-1])

                for sg in mso_schema.schema_objects["site_service_graph"].details.get("serviceNodes", []):
                    if device == sg.get("device").get("dn").split("/")[-1].split("-")[-1]:
                        parent_object = dict(deviceConfiguration=dict(cloudLoadBalancer=dict(listeners=[])), serviceNodeRef=sg.get("serviceNodeRef"))
                        break
            else:
                mso.fail_json(
                    msg="Listener is not supported for the 'service_node_index': {0} is associated with the Third-Party {1} device.".format(
                        service_node_index, "Load Balancer" if query_device_data.get("deviceVendorType") == "ADC" else "Firewall"
                    )
                )
        # Parent object creation logic ends

        # Listener object creation logic begins
        listener_object = dict(
            name=listener,
            protocol=listener_protocol,
            port=listener_port,
            secPolicy=security_policy,
        )

        if frontend_ip:
            mso.input_validation("frontend_ip", frontend_ip, ["tenant", "device"], module.params, None, object_name=listener)
            listener_object["nlbDevIp"] = dict(name=frontend_ip, dn="uni/tn-{0}/clb-{1}/vip-{2}".format(tenant, device, frontend_ip))

        if ssl_certificates:
            listener_object["certificates"] = [
                {
                    "name": ssl_certificate.get("name"),
                    "tDn": "uni/tn-{0}/certstore".format(tenant),
                    "default": True,
                    "store": ssl_certificate.get("certificate_store"),
                }
                for ssl_certificate in ssl_certificates
            ]

        # Rules object creation logic
        rules_data = []

        for position, rule in enumerate(rules, 0):
            if (listener_protocol == "http" and rule.get("protocol") == "http") or (listener_protocol == "https" and rule.get("protocol") == "https"):
                mso.fail_json(msg="When the 'listener_protocol' is '{0}', the rule 'protocol' must be '{1}'".format(listener_protocol, rule.get("protocol")))

            if rule.get("action_type") == "redirect":
                mso.input_validation(
                    "action_type", "redirect", ["redirect_protocol", "redirect_port", "url_type", "redirect_code"], rule, position, rule.get("name")
                )
            elif rule.get("action_type") == "forward":
                mso.input_validation("action_type", "forward", ["protocol", "port", "health_check"], rule, position, rule.get("name"))

            if rule.get("url_type") == "custom":
                mso.input_validation(
                    "url_type", "custom", ["redirect_host_name", "redirect_path", "redirect_query", "response_code"], rule, position, rule.get("name")
                )

            rule_data = dict(
                name=rule.get("name"),
                floatingIp=rule.get("floating_ip"),
                index=rule.get("priority"),
                host=rule.get("host"),
                path=rule.get("path"),
                action=rule.get("action"),
                actionType=LISTENER_ACTION_TYPE_MAP.get(rule.get("action_type")),
                contentType=LISTENER_CONTENT_TYPE_MAP.get(rule.get("content_type")),
                port=rule.get("port"),
                protocol=rule.get("protocol"),
                urlType=rule.get("url_type"),
                customURL=rule.get("custom_url"),
                redirectHostName=rule.get("redirect_host_name"),
                redirectPath=rule.get("redirect_path"),
                redirectQuery=rule.get("redirect_query"),
                responseCode=rule.get("response_code"),
                responseBody=rule.get("response_body"),
                redirectProtocol=rule.get("redirect_protocol"),
                redirectPort=rule.get("redirect_port"),
                redirectCode=LISTENER_REDIRECT_CODE_MAP.get(rule.get("redirect_code")),
                targetIpType=rule.get("target_ip_type"),
            )

            if listener_protocol in ["tcp", "udp"]:
                mso.input_validation("listener_protocol", "tcp/udp", ["health_check"], rule)

            provider_epg = rule.get("provider_epg")
            if provider_epg:
                rule_data["providerEpgRef"] = "/schemas/{0}/templates/{1}/anps/{2}/epgs/{3}".format(
                    provider_epg.get("schema") or schema_id,
                    provider_epg.get("template_name") or template,
                    provider_epg.get("anp_name"),
                    provider_epg.get("epg_name"),
                )

            health_check = rule.get("health_check")
            if health_check:
                if listener_protocol in ["tcp", "udp"]:
                    if health_check.get("protocol") == "tcp":
                        mso.input_validation("health_check - 'protocol'", "tcp", ["port", "unhealthy_threshold", "interval"], health_check)
                    elif health_check.get("protocol") in ["http", "https"]:
                        mso.input_validation("health_check - 'protocol'", "http/https", ["port", "path", "unhealthy_threshold", "interval"], health_check)
                elif (listener_protocol == "http" and health_check.get("protocol") == "https") or (
                    listener_protocol == "https" and health_check.get("protocol") == "http"
                ):
                    mso.input_validation(
                        "health_check - 'protocol'", "http/https", ["port", "path", "unhealthy_threshold", "timeout", "interval"], health_check
                    )
                else:
                    mso.fail_json(
                        msg=(
                            "The 'listener_protocol': {0} and the health_check protocol: {1} "
                            + "is not a valid configuration at the object position: {2} and the object name: {3}"
                        ).format(listener_protocol, health_check.get("protocol"), position, rule.get("name"))
                    )

                health_check_data = dict(
                    port=health_check.get("port"),
                    protocol=health_check.get("protocol"),
                    path=health_check.get("path"),
                    interval=health_check.get("interval"),
                    timeout=health_check.get("timeout"),
                    unhealthyThreshold=health_check.get("unhealthy_threshold"),
                    successCode=health_check.get("success_code"),
                    useHostFromRule=YES_OR_NO_TO_BOOL_STRING_MAP.get(health_check.get("use_host_from_rule")),
                    host=health_check.get("host"),
                )

                rule_data["healthCheck"] = health_check_data

            rules_data.append(rule_data)

        listener_object["rules"] = rules_data

        if parent_present:
            # Update an existing listener
            if mso.existing:
                listener_path = (
                    "/sites/{0}-{1}/contracts/{2}/serviceGraphRelationship/serviceNodesRelationship/{3}/deviceConfiguration/cloudLoadBalancer/listeners/{4}"
                ).format(site_id, template, contract, service_node_index, listener)
                op = "replace"
            else:
                # Create a new listener
                listener_path = (
                    "/sites/{0}-{1}/contracts/{2}/serviceGraphRelationship/serviceNodesRelationship/{3}/deviceConfiguration/cloudLoadBalancer/listeners/-"
                ).format(site_id, template, contract, service_node_index)
                op = "add"
            parent_object = listener_object
        else:
            # Create a new listener with parent object
            listener_path = "/sites/{0}-{1}/contracts/{2}/serviceGraphRelationship/serviceNodesRelationship/{3}".format(
                site_id, template, contract, service_node_index
            )
            op = "replace"
            parent_object["deviceConfiguration"]["cloudLoadBalancer"]["listeners"].append(listener_object)

        mso.sanitize(parent_object, collate=True)
        ops.append(dict(op=op, path=listener_path, value=mso.sent))

    elif state == "absent":
        if mso.existing:
            listener_path = (
                "/sites/{0}-{1}/contracts/{2}/serviceGraphRelationship/serviceNodesRelationship/{3}/deviceConfiguration/cloudLoadBalancer/listeners/{4}"
            ).format(site_id, template, contract, service_node_index, listener)
            ops.append(dict(op="remove", path=listener_path))

    mso.existing = mso.proposed

    if not module.check_mode:
        mso.request(mso_schema.path, method="PATCH", data=ops)

    mso.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2018, Dag Wieers (@dagwieers) <dag@wieers.com>
# Copyright: (c) 2020, Shreyas Srish (@shrsr) <ssrish@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: mso_tenant_site
short_description: Manage tenants with cloud sites.
description:
- Manage tenants with cloud sites on Cisco ACI Multi-Site.
author:
- Shreyas Srish (@shrsr)
options:
  tenant:
    description:
    - The name of the tenant.
    type: str
    required: true
  site:
    description:
    - The name of the site.
    - This can either be cloud site or non-cloud site.
    type: str
    aliases: [ name ]
  cloud_account:
    description:
    - Required for cloud site.
    - Account id of AWS in the form '000000000000'.
    - Account id of Azure in the form 'uni/tn-(tenant_name)/act-[(subscription_id)]-azure_vendor-azure'.
    - Example values inside account id of Azure '(tenant_name)=tenant_test and (subscription_id)=10'.
    type: str
  security_domains:
    description:
    - List of security domains for sites.
    type: list
    elements: str
    default: []
  aws_account_org:
    description:
    - AWS account for organization.
    default: false
    type: bool
  aws_trusted:
    description:
    - AWS account's access in trusted mode. Credentials are required, when set to false.
    type: bool
  aws_access_key:
    description:
    - AWS account's access key id. This is required when aws_trusted is set to false.
    type: str
  azure_access_type:
    description:
    - Managed mode for Azure.
    - Unmanaged mode for Azure.
    - Shared mode if the attribute is not specified.
    choices: [ managed, unmanaged, shared ]
    default: shared
    type: str
  azure_active_directory_id:
    description:
    - Azure account's active directory id.
    - This attribute is required when azure_access_type is in unmanaged mode.
    type: str
  azure_active_directory_name:
    description:
    - Azure account's active directory name. Example being 'CiscoINSBUAd' as active directory name.
    - This attribute is required when azure_access_type is in unmanaged mode.
    type: str
  azure_subscription_id:
    description:
    - Azure account's subscription id.
    - This attribute is required when azure_access_type is either in managed mode or unmanaged mode.
    type: str
  azure_application_id:
    description:
    - Azure account's application id.
    - This attribute is required when azure_access_type is either in managed mode or unmanaged mode.
    type: str
  azure_credential_name:
    description:
    - Azure account's credential name.
    - This attribute is required when azure_access_type is in unmanaged mode.
    type: str
  secret_key:
    description:
    - secret key of AWS for untrusted account. Required when aws_trusted is set to false.
    - secret key of Azure account for unmanaged identity. Required in unmanaged mode of Azure account.
    type: str
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Associate a non-cloud site with a tenant
  cisco.mso.mso_tenant_site:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    tenant: tenant_name
    site: site_name
    state: present

- name: Associate AWS site with a tenant, with aws_trusted set to true
  cisco.mso.mso_tenant_site:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    tenant: tenant_name
    site: site_name
    cloud_account: '000000000000'
    aws_trusted: true
    state: present

- name: Associate AWS site with a tenant, with aws_trusted set to false
  cisco.mso.mso_tenant_site:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    tenant: tenant_name
    site: AWS
    cloud_account: '000000000000'
    aws_trusted: false
    aws_access_key: '1'
    secret_key: '0'
    aws_account_org: false
    state: present

- name: Associate Azure site in managed mode
  mso.cisco.mso_tenant_site:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    tenant: tenant_name
    site: site_name
    cloud_account: uni/tn-ansible_test/act-[9]-azure_vendor-azure
    azure_access_type: managed
    azure_subscription_id: '9'
    azure_application_id: '100'
    state: present

- name: Associate Azure site in unmanaged mode
  mso.cisco.mso_tenant_site:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    tenant: tenant_name
    site: site_name
    cloud_account: uni/tn-ansible_test/act-[9]-azure_vendor-azure
    azure_access_type: unmanaged
    azure_subscription_id: '9'
    azure_application_id: '100'
    azure_credential_name: cApicApp
    secret_key: iins
    azure_active_directory_id: '32'
    azure_active_directory_name: CiscoINSBUAd
    state: present

- name: Dissociate a site
  cisco.mso.mso_tenant_site:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    tenant: tenant_name
    site: site_name
    state: absent

- name: Query a site
  cisco.mso.mso_tenant_site:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    tenant: tenant_name
    site: site_name
    state: query
  register: query_result

- name: Query all sites of a tenant
  cisco.mso.mso_tenant_site:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    tenant: tenant_name
    state: query
  register: query_result
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        tenant=dict(type="str", required=True),
        site=dict(type="str", aliases=["name"]),
        cloud_account=dict(type="str"),
        security_domains=dict(type="list", elements="str", default=[]),
        aws_trusted=dict(type="bool"),
        azure_access_type=dict(type="str", default="shared", choices=["managed", "unmanaged", "shared"]),
        azure_active_directory_id=dict(type="str"),
        aws_access_key=dict(type="str", no_log=True),
        aws_account_org=dict(type="bool", default="false"),
        azure_active_directory_name=dict(type="str"),
        azure_subscription_id=dict(type="str"),
        azure_application_id=dict(type="str"),
        azure_credential_name=dict(type="str"),
        secret_key=dict(type="str", no_log=True),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["tenant", "site"]],
            ["state", "present", ["tenant", "site"]],
        ],
    )

    state = module.params.get("state")
    security_domains = module.params.get("security_domains")
    cloud_account = module.params.get("cloud_account")
    azure_access_type = module.params.get("azure_access_type")
    azure_credential_name = module.params.get("azure_credential_name")
    azure_application_id = module.params.get("azure_application_id")
    azure_active_directory_id = module.params.get("azure_active_directory_id")
    azure_active_directory_name = module.params.get("azure_active_directory_name")
    azure_subscription_id = module.params.get("azure_subscription_id")
    secret_key = module.params.get("secret_key")
    aws_account_org = module.params.get("aws_account_org")
    aws_access_key = module.params.get("aws_access_key")
    aws_trusted = module.params.get("aws_trusted")

    mso = MSOModule(module)

    # Get tenant_id and site_id
    tenant_id = mso.lookup_tenant(module.params.get("tenant"))

    # To ignore the object not found issue for the lookup methods
    site_id = mso.lookup_site(module.params.get("site"), True)

    if state == "absent" and not site_id:
        mso.exit_json()

    tenants = [(t.get("id")) for t in mso.query_objs("tenants")]
    tenant_idx = tenants.index((tenant_id))

    # set tenent and port paths
    tenant_path = "tenants/{0}".format(tenant_id)
    ops = []
    ports_path = "/siteAssociations/-"
    port_path = "/siteAssociations/{0}".format(site_id)

    payload = dict(
        siteId=site_id,
        securityDomains=security_domains,
        cloudAccount=cloud_account,
    )

    if cloud_account:
        if "azure" in cloud_account:
            azure_account = dict(
                accessType=azure_access_type,
                securityDomains=security_domains,
                vendor="azure",
            )

            payload["azureAccount"] = [azure_account]

            cloudSubscription = dict(
                cloudSubscriptionId=azure_subscription_id,
                cloudApplicationId=azure_application_id,
            )

            payload["azureAccount"][0]["cloudSubscription"] = cloudSubscription

            if azure_access_type == "shared":
                payload["azureAccount"] = []

            if azure_access_type == "managed":
                if not azure_subscription_id:
                    mso.fail_json(msg="azure_susbscription_id is required when in managed mode.")
                if not azure_application_id:
                    mso.fail_json(msg="azure_application_id is required when in managed mode.")
                payload["azureAccount"][0]["cloudApplication"] = []
                payload["azureAccount"][0]["cloudActiveDirectory"] = []

            if azure_access_type == "unmanaged":
                if not azure_subscription_id:
                    mso.fail_json(msg="azure_subscription_id is required when in unmanaged mode.")
                if not azure_application_id:
                    mso.fail_json(msg="azure_application_id is required when in unmanaged mode.")
                if not secret_key:
                    mso.fail_json(msg="secret_key is required when in unmanaged mode.")
                if not azure_active_directory_id:
                    mso.fail_json(msg="azure_active_directory_id is required when in unmanaged mode.")
                if not azure_active_directory_name:
                    mso.fail_json(msg="azure_active_directory_name is required when in unmanaged mode.")
                if not azure_credential_name:
                    mso.fail_json(msg="azure_credential_name is required when in unmanaged mode.")
                azure_account.update(
                    accessType="credentials",
                )
                cloudApplication = dict(
                    cloudApplicationId=azure_application_id,
                    cloudCredentialName=azure_credential_name,
                    secretKey=secret_key,
                    cloudActiveDirectoryId=azure_active_directory_id,
                )
                cloudActiveDirectory = dict(cloudActiveDirectoryId=azure_active_directory_id, cloudActiveDirectoryName=azure_active_directory_name)
                payload["azureAccount"][0]["cloudApplication"] = [cloudApplication]
                payload["azureAccount"][0]["cloudActiveDirectory"] = [cloudActiveDirectory]

        else:
            aws_account = dict(
                accountId=cloud_account,
                isTrusted=aws_trusted,
                accessKeyId=aws_access_key,
                secretKey=secret_key,
                isAccountInOrg=aws_account_org,
            )

            if not aws_trusted:
                if not aws_access_key:
                    mso.fail_json(msg="aws_access_key is a required field in untrusted mode.")
                if not secret_key:
                    mso.fail_json(msg="secret_key is a required field in untrusted mode.")
            payload["awsAccount"] = [aws_account]

    sites = [(s.get("siteId")) for s in mso.query_objs("tenants")[tenant_idx]["siteAssociations"]]

    if site_id in sites:
        site_idx = sites.index((site_id))
        mso.existing = mso.query_objs("tenants")[tenant_idx]["siteAssociations"][site_idx]

    if state == "query":
        if len(sites) == 0:
            mso.fail_json(msg="No site associated with tenant Id {0}".format(tenant_id))
        elif site_id not in sites and site_id is not None:
            mso.fail_json(msg="Site Id {0} not associated with tenant Id {1}".format(site_id, tenant_id))
        elif site_id is None:
            mso.existing = mso.query_objs("tenants")[tenant_idx]["siteAssociations"]
        mso.exit_json()

    mso.previous = mso.existing

    if state == "absent":
        if mso.existing:
            mso.sent = mso.existing = {}
            ops.append(dict(op="remove", path=port_path))
    if state == "present":
        mso.sanitize(payload, collate=True)

        if mso.existing:
            ops.append(dict(op="replace", path=port_path, value=mso.sent))
        else:
            ops.append(dict(op="add", path=ports_path, value=mso.sent))

        mso.existing = mso.proposed

    if not module.check_mode and mso.proposed != mso.previous:
        mso.request(tenant_path, method="PATCH", data=ops)

    mso.exit_json()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Copyright 2024 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the
# ansible.content_builder.
#
# Manually editing this file is not advised.
#
# To update the argspec make the desired changes
# in the documentation in the module file and re-run
# ansible.content_builder commenting out
# the path to external 'docstring' in build.yaml.
#
##############################################

"""
The arg spec for the nxos_vrf_global module
"""


class Vrf_globalArgs(object):  # pylint: disable=R0903
    """The arg spec for the nxos_vrf_global module"""

    argument_spec = {
        "config": {
            "type": "dict",
            "options": {
                "vrfs": {
                    "type": "list",
                    "elements": "dict",
                    "options": {
                        "name": {"required": True, "type": "str"},
                        "description": {"type": "str"},
                        "rd": {"type": "str"},
                        "ip": {
                            "type": "dict",
                            "options": {
                                "auto_discard": {"type": "bool"},
                                "domain_list": {"type": "list", "elements": "str"},
                                "domain_name": {"type": "str"},
                                "icmp_err": {
                                    "type": "dict",
                                    "options": {
                                        "source_interface": {
                                            "type": "dict",
                                            "options": {
                                                "interface": {
                                                    "type": "str",
                                                    "choices": [
                                                        "loopback",
                                                        "ethernet",
                                                        "port-channel",
                                                    ],
                                                },
                                                "interface_value": {"type": "str"},
                                            },
                                        },
                                    },
                                },
                                "igmp": {
                                    "type": "dict",
                                    "options": {
                                        "ssm_translate": {
                                            "type": "list",
                                            "elements": "dict",
                                            "options": {
                                                "group": {"type": "str"},
                                                "source": {"type": "str"},
                                            },
                                        },
                                    },
                                },
                                "mroutes": {
                                    "type": "list",
                                    "elements": "dict",
                                    "options": {
                                        "group": {"type": "str"},
                                        "source": {"type": "str"},
                                        "preference": {"type": "int"},
                                        "vrf": {"type": "str"},
                                    },
                                },
                                "multicast": {
                                    "type": "dict",
                                    "options": {
                                        "group_range_prefix_list": {"type": "str"},
                                        "multipath": {
                                            "type": "dict",
                                            "options": {
                                                "resilient": {"type": "bool"},
                                                "splitting_type": {
                                                    "type": "dict",
                                                    "mutually_exclusive": [
                                                        [
                                                            "legacy",
                                                            "nbm",
                                                            "none",
                                                            "sg_hash",
                                                            "sg_hash_next_hop",
                                                        ],
                                                    ],
                                                    "options": {
                                                        "none": {"type": "bool"},
                                                        "legacy": {"type": "bool"},
                                                        "nbm": {"type": "bool"},
                                                        "sg_hash": {
                                                            "type": "bool",
                                                        },
                                                        "sg_hash_next_hop": {
                                                            "type": "bool",
                                                        },
                                                    },
                                                },
                                            },
                                        },
                                        "rpf": {
                                            "type": "list",
                                            "elements": "dict",
                                            "options": {
                                                "vrf_name": {"type": "str"},
                                                "group_list_range": {
                                                    "type": "str",
                                                },
                                            },
                                        },
                                    },
                                },
                                "name_server": {
                                    "type": "dict",
                                    "options": {
                                        "address_list": {
                                            "type": "list",
                                            "elements": "str",
                                        },
                                        "use_vrf": {
                                            "type": "dict",
                                            "options": {
                                                "vrf": {"type": "str"},
                                                "source_address": {"type": "str"},
                                            },
                                        },
                                    },
                                },
                                "route": {
                                    "type": "list",
                                    "elements": "dict",
                                    "mutually_exclusive": [
                                        ["tags", "vrf", "track"],
                                    ],
                                    "options": {
                                        "source": {"type": "str"},
                                        "destination": {"type": "str"},
                                        "tags": {
                                            "type": "dict",
                                            "options": {
                                                "tag_value": {"type": "int"},
                                                "route_pref": {"type": "int"},
                                            },
                                        },
                                        "vrf": {"type": "str"},
                                        "track": {"type": "str"},
                                    },
                                },
                            },
                        },
                        "vni": {
                            "type": "dict",
                            "options": {
                                "vni_number": {"type": "int"},
                                "layer_3": {"type": "bool"},
                            },
                        },
                        "multicast": {
                            "type": "dict",
                            "options": {
                                "service_reflect": {
                                    "type": "list",
                                    "elements": "dict",
                                    "options": {
                                        "service_interface": {"type": "str"},
                                        "map_to": {"type": "str"},
                                    },
                                },
                            },
                        },
                        "ipv6": {
                            "type": "dict",
                            "options": {
                                "mld_ssm_translate": {
                                    "type": "list",
                                    "elements": "dict",
                                    "options": {
                                        "icmp": {"type": "bool"},
                                        "group": {"type": "str"},
                                        "source": {"type": "str"},
                                    },
                                },
                                "multicast": {
                                    "type": "dict",
                                    "options": {
                                        "group_range_prefix_list": {"type": "str"},
                                        "multipath": {
                                            "type": "dict",
                                            "options": {
                                                "resilient": {"type": "bool"},
                                                "splitting_type": {
                                                    "type": "dict",
                                                    "mutually_exclusive": [
                                                        [
                                                            "none",
                                                            "sg_hash",
                                                            "sg_hash_next_hop",
                                                        ],
                                                    ],
                                                    "options": {
                                                        "none": {"type": "bool"},
                                                        "sg_hash": {
                                                            "type": "bool",
                                                        },
                                                        "sg_hash_next_hop": {
                                                            "type": "bool",
                                                        },
                                                    },
                                                },
                                            },
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        "running_config": {"type": "str"},
        "state": {
            "choices": [
                "parsed",
                "gathered",
                "deleted",
                "merged",
                "replaced",
                "rendered",
                "overridden",
                "purged",
            ],
            "default": "merged",
            "type": "str",
        },
    }  # pylint: disable=C0301



# Copyright (c) 2015, Björn Andersson
# Copyright (c) 2021, Ansible Project
# Copyright (c) 2021, Abhijeet Kasurde <akasurde@redhat.com>
# Copyright (c) 2022, Alexei Znamensky <russoz@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import os


def determine_config_file(user: str | None, config_file: str | None) -> str:
    if user:
        config_file = os.path.join(os.path.expanduser(f"~{user}"), ".ssh", "config")
    elif config_file is None:
        config_file = "/etc/ssh/ssh_config"
    return config_file



#!/usr/bin/python

# Copyright (c) 2023, Salvatore Mesoraca <s.mesoraca16@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
from __future__ import annotations

DOCUMENTATION = r"""
module: kdeconfig
short_description: Manage KDE configuration files
version_added: "6.5.0"
description:
  - Add or change individual settings in KDE configuration files.
  - It uses B(kwriteconfig) under the hood.
options:
  path:
    description:
      - Path to the config file. If the file does not exist it is created.
    type: path
    required: true
  kwriteconfig_path:
    description:
      - Path to the kwriteconfig executable. If not specified, Ansible tries to discover it.
    type: path
  values:
    description:
      - List of values to set.
    type: list
    elements: dict
    suboptions:
      group:
        description:
          - The option's group. One between this and O(values[].groups) is required.
        type: str
      groups:
        description:
          - List of the option's groups. One between this and O(values[].group) is required.
        type: list
        elements: str
      key:
        description:
          - The option's name.
        type: str
        required: true
      value:
        description:
          - The option's value. One between this and O(values[].bool_value) is required.
        type: str
      bool_value:
        description:
          - Boolean value.
          - One between this and O(values[].value) is required.
        type: bool
    required: true
  backup:
    description:
      - Create a backup file.
    type: bool
    default: false
extends_documentation_fragment:
  - ansible.builtin.files
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: full
requirements:
  - kwriteconfig
author:
  - Salvatore Mesoraca (@smeso)
"""

EXAMPLES = r"""
- name: Ensure "Homepage=https://www.ansible.com/" in group "Branding"
  community.general.kdeconfig:
    path: /etc/xdg/kickoffrc
    values:
      - group: Branding
        key: Homepage
        value: https://www.ansible.com/
    mode: '0644'

- name: Ensure "KEY=true" in groups "Group" and "Subgroup", and "KEY=VALUE" in Group2
  community.general.kdeconfig:
    path: /etc/xdg/someconfigrc
    values:
      - groups: [Group, Subgroup]
        key: KEY
        bool_value: true
      - group: Group2
        key: KEY
        value: VALUE
    backup: true
"""

RETURN = r""" # """

import os
import shutil
import tempfile
import traceback

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.text.converters import to_bytes, to_text


class TemporaryDirectory:
    """Basic backport of tempfile.TemporaryDirectory"""

    def __init__(self, suffix="", prefix="tmp", dir=None):
        self.name = None
        self.name = tempfile.mkdtemp(suffix, prefix, dir)

    def __enter__(self):
        return self.name

    def rm(self):
        if self.name:
            shutil.rmtree(self.name, ignore_errors=True)
            self.name = None

    def __exit__(self, exc, value, tb):
        self.rm()

    def __del__(self):
        self.rm()


def run_kwriteconfig(module, cmd, path, groups, key, value):
    """Invoke kwriteconfig with arguments"""
    args = [cmd, "--file", path, "--key", key]
    for group in groups:
        args.extend(["--group", group])
    if isinstance(value, bool):
        args.extend(["--type", "bool"])
        if value:
            args.append("true")
        else:
            args.append("false")
    else:
        args.extend(["--", value])
    module.run_command(args, check_rc=True)


def run_module(module, tmpdir, kwriteconfig):
    result = dict(changed=False, msg="OK", path=module.params["path"])
    b_path = to_bytes(module.params["path"])
    tmpfile = os.path.join(tmpdir, "file")
    b_tmpfile = to_bytes(tmpfile)
    diff = dict(
        before="",
        after="",
        before_header=result["path"],
        after_header=result["path"],
    )
    try:
        with open(b_tmpfile, "wb") as dst:
            try:
                with open(b_path, "rb") as src:
                    b_data = src.read()
            except OSError:
                result["changed"] = True
            else:
                dst.write(b_data)
                try:
                    diff["before"] = to_text(b_data)
                except UnicodeError:
                    diff["before"] = repr(b_data)
    except OSError:
        module.fail_json(msg="Unable to create temporary file", traceback=traceback.format_exc())

    for row in module.params["values"]:
        groups = row["groups"]
        if groups is None:
            groups = [row["group"]]
        key = row["key"]
        value = row["bool_value"]
        if value is None:
            value = row["value"]
        run_kwriteconfig(module, kwriteconfig, tmpfile, groups, key, value)

    with open(b_tmpfile, "rb") as tmpf:
        b_data = tmpf.read()
        try:
            diff["after"] = to_text(b_data)
        except UnicodeError:
            diff["after"] = repr(b_data)

    result["changed"] = result["changed"] or diff["after"] != diff["before"]

    file_args = module.load_file_common_arguments(module.params)

    if module.check_mode:
        if not result["changed"]:
            shutil.copystat(b_path, b_tmpfile)
            uid, gid = module.user_and_group(b_path)
            os.chown(b_tmpfile, uid, gid)
            if module._diff:
                diff = {}
            else:
                diff = None
            result["changed"] = module.set_fs_attributes_if_different(file_args, result["changed"], diff=diff)
        if module._diff:
            result["diff"] = diff
        module.exit_json(**result)

    if result["changed"]:
        if module.params["backup"] and os.path.exists(b_path):
            result["backup_file"] = module.backup_local(result["path"])
        try:
            module.atomic_move(b_tmpfile, os.path.abspath(b_path))
        except OSError:
            module.ansible.fail_json(
                msg=f"Unable to move temporary file {tmpfile} to {result['path']}, IOError",
                traceback=traceback.format_exc(),
            )

    if result["changed"]:
        module.set_fs_attributes_if_different(file_args, result["changed"])
    else:
        if module._diff:
            diff = {}
        else:
            diff = None
        result["changed"] = module.set_fs_attributes_if_different(file_args, result["changed"], diff=diff)
    if module._diff:
        result["diff"] = diff
    module.exit_json(**result)


def main():
    single_value_arg = dict(
        group=dict(type="str"),
        groups=dict(type="list", elements="str"),
        key=dict(type="str", required=True, no_log=False),
        value=dict(type="str"),
        bool_value=dict(type="bool"),
    )
    required_alternatives = [("group", "groups"), ("value", "bool_value")]
    module_args = dict(
        values=dict(
            type="list",
            elements="dict",
            options=single_value_arg,
            mutually_exclusive=required_alternatives,
            required_one_of=required_alternatives,
            required=True,
        ),
        path=dict(type="path", required=True),
        kwriteconfig_path=dict(type="path"),
        backup=dict(type="bool", default=False),
    )

    module = AnsibleModule(
        argument_spec=module_args,
        add_file_common_args=True,
        supports_check_mode=True,
    )

    kwriteconfig = None
    if module.params["kwriteconfig_path"] is not None:
        kwriteconfig = module.get_bin_path(module.params["kwriteconfig_path"], required=True)
    else:
        for progname in ("kwriteconfig6", "kwriteconfig5", "kwriteconfig", "kwriteconfig4"):
            kwriteconfig = module.get_bin_path(progname)
            if kwriteconfig is not None:
                break
        if kwriteconfig is None:
            module.fail_json(msg="kwriteconfig is not installed")
    for v in module.params["values"]:
        if not v["key"]:
            module.fail_json(msg="'key' cannot be empty")
    with TemporaryDirectory(dir=module.tmpdir) as tmpdir:
        run_module(module, tmpdir, kwriteconfig)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: ec2_vpc_nat_gateway
version_added: 1.0.0
short_description: Manage AWS VPC NAT Gateways
description:
  - Ensure the state of AWS VPC NAT Gateways based on their id, allocation and subnet ids.
options:
  state:
    description:
      - Ensure NAT Gateway is present or absent.
    default: "present"
    choices: ["present", "absent"]
    type: str
  nat_gateway_id:
    description:
      - The id AWS dynamically allocates to the NAT Gateway on creation.
        This is required when the absent option is present.
    type: str
  subnet_id:
    description:
      - The id of the subnet to create the NAT Gateway in. This is required
        with the present option.
    type: str
  allocation_id:
    description:
      - The id of the elastic IP allocation. If this is not passed and the
        eip_address is not passed. An EIP is generated for this NAT Gateway.
    type: str
  connectivity_type:
    description:
      - Indicates whether the NAT gateway supports public or private connectivity.
    choices: ["public", "private"]
    default: "public"
    type: str
    version_added: 5.5.0
  eip_address:
    description:
      - The elastic IP address of the EIP you want attached to this NAT Gateway.
        If this is not passed and the O(allocation_id) is not passed,
        an EIP is generated for this NAT Gateway.
    type: str
  if_exist_do_not_create:
    description:
      - If a NAT Gateway exists already in the O(subnet_id), then do not create a new one.
    required: false
    default: false
    type: bool
  release_eip:
    description:
      - Deallocate the EIP from the VPC.
      - Option is only valid with the absent state.
      - You should use this with the wait option. Since you can not release an address while a delete operation is happening.
    default: false
    type: bool
  wait:
    description:
      - Wait for operation to complete before returning.
    default: false
    type: bool
  wait_timeout:
    description:
      - How many seconds to wait for an operation to complete before timing out.
    default: 320
    type: int
  client_token:
    description:
      - Optional unique token to be used during create to ensure idempotency.
        When specifying this option, ensure you specify the eip_address parameter
        as well otherwise any subsequent runs will fail.
    type: str
  default_create:
    description:
      - When O(default_create=true) and O(eip_address) has been set, but not yet
        allocated, the NAT gateway is created and a new EIP is automatically allocated.
      - When O(default_create=false) and O(eip_address) has been set, but not yet
        allocated, the module will fail.
      - If O(eip_address) has not been set, this parameter has no effect.
    default: false
    type: bool
    version_added: 6.2.0
author:
  - Allen Sanabria (@linuxdynasty)
  - Jon Hadfield (@jonhadfield)
  - Karen Cheng (@Etherdaemon)
  - Alina Buzachis (@alinabuzachis)
notes:
  - Support for O(tags) and O(purge_tags) was added in release 1.4.0.
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.tags
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

- name: Create new nat gateway with client token.
  amazon.aws.ec2_vpc_nat_gateway:
    state: present
    subnet_id: subnet-12345678
    eip_address: 52.1.1.1
    region: ap-southeast-2
    client_token: abcd-12345678
  register: new_nat_gateway

- name: Create new nat gateway using an allocation-id and connectivity type.
  amazon.aws.ec2_vpc_nat_gateway:
    state: present
    subnet_id: subnet-12345678
    allocation_id: eipalloc-12345678
    connectivity_type: "private"
    region: ap-southeast-2
  register: new_nat_gateway

- name: Create new nat gateway, using an EIP address  and wait for available status.
  amazon.aws.ec2_vpc_nat_gateway:
    state: present
    subnet_id: subnet-12345678
    eip_address: 52.1.1.1
    wait: true
    region: ap-southeast-2
  register: new_nat_gateway

- name: Create new nat gateway and allocate new EIP.
  amazon.aws.ec2_vpc_nat_gateway:
    state: present
    subnet_id: subnet-12345678
    wait: true
    region: ap-southeast-2
  register: new_nat_gateway

- name: Create new nat gateway and allocate new EIP if a nat gateway does not yet exist in the subnet.
  amazon.aws.ec2_vpc_nat_gateway:
    state: present
    subnet_id: subnet-12345678
    wait: true
    region: ap-southeast-2
    if_exist_do_not_create: true
  register: new_nat_gateway

- name: Delete nat gateway using discovered nat gateways from facts module.
  amazon.aws.ec2_vpc_nat_gateway:
    state: absent
    region: ap-southeast-2
    wait: true
    nat_gateway_id: "{{ item.NatGatewayId }}"
    release_eip: true
  register: delete_nat_gateway_result
  loop: "{{ gateways_to_remove.result }}"

- name: Delete nat gateway and wait for deleted status.
  amazon.aws.ec2_vpc_nat_gateway:
    state: absent
    nat_gateway_id: nat-12345678
    wait: true
    wait_timeout: 500
    region: ap-southeast-2

- name: Delete nat gateway and release EIP.
  amazon.aws.ec2_vpc_nat_gateway:
    state: absent
    nat_gateway_id: nat-12345678
    release_eip: true
    wait: true
    wait_timeout: 300
    region: ap-southeast-2

- name: Create new nat gateway using allocation-id and tags.
  amazon.aws.ec2_vpc_nat_gateway:
    state: present
    subnet_id: subnet-12345678
    allocation_id: eipalloc-12345678
    region: ap-southeast-2
    tags:
      Tag1: tag1
      Tag2: tag2
  register: new_nat_gateway

- name: Update tags without purge
  amazon.aws.ec2_vpc_nat_gateway:
    subnet_id: subnet-12345678
    allocation_id: eipalloc-12345678
    region: ap-southeast-2
    purge_tags: false
    tags:
      Tag3: tag3
    wait: true
  register: update_tags_nat_gateway
"""

RETURN = r"""
connectivity_type:
    description:
      - Indicates whether the NAT gateway supports public or private connectivity.
    returned: always
    type: str
    sample: public
create_time:
  description: The ISO 8601 date time format in UTC.
  returned: always
  type: str
  sample: "2016-03-05T05:19:20.282000+00:00'"
nat_gateway_id:
  description: Id of the VPC NAT Gateway.
  returned: always
  type: str
  sample: "nat-0d1e3a878585988f8"
subnet_id:
  description: Id of the Subnet.
  returned: always
  type: str
  sample: "subnet-12345"
state:
  description: The current state of the NAT Gateway.
  returned: always
  type: str
  sample: "available"
tags:
  description: The tags associated the VPC NAT Gateway.
  type: dict
  returned: When tags are present.
  sample:
    tags:
        "Ansible": "Test"
vpc_id:
  description: Id of the VPC.
  returned: always
  type: str
  sample: "vpc-12345"
nat_gateway_addresses:
  description: List of dictionaries containing the public_ip, network_interface_id, private_ip, and allocation_id.
  returned: always
  type: complex
  contains:
    allocation_id:
        description: The allocation ID of the Elastic IP address that's associated with the NAT gateway.
        returned: always
        type: str
        sample: eipalloc-0853e66a40803da76
    association_id:
        description: The association ID of the Elastic IP address that is associated with the NAT gateway.
        returned: always
        type: str
        sample: eipassoc-0d6365c7eeb7d4932
    is_primary:
        description: Defines if the IP address is the primary address.
        returned: always
        type: bool
        sample: true
    network_interface_id:
        description: The ID of the network interface associated with the NAT gateway.
        returned: always
        type: str
        sample: eni-0a37acdbe306c661c
    private_ip:
        description: The private IP address associated with the Elastic IP address.
        returned: always
        type: str
        sample: 10.0.238.227
    public_ip:
        description: The Elastic IP address associated with the NAT gateway.
        returned: always
        type: str
        sample: 34.204.123.52
    status:
        description: The address status.
        returned: always
        type: str
        sample: succeeded
  sample: [
       {
            "allocation_id": "eipalloc-08ec128d03629671d",
            "association_id": "eipassoc-0d6365c7eeb7d4932",
            "is_primary": true,
            "network_interface_id": "eni-095104e630881bad6",
            "private_ip": "10.1.0.250",
            "public_ip": "34.202.90.172",
            "status": "succeeded"
        }
  ]
"""

import datetime
from typing import Any
from typing import Dict
from typing import List
from typing import Optional
from typing import Tuple

try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.ec2 import AnsibleEC2Error
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import allocate_address
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import create_nat_gateway
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import delete_nat_gateway
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_addresses
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_ec2_tags
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_nat_gateways
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import ensure_ec2_tags
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import release_address
from ansible_collections.amazon.aws.plugins.module_utils.exceptions import AnsibleAWSError
from ansible_collections.amazon.aws.plugins.module_utils.exceptions import is_ansible_aws_error_code
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_specifications
from ansible_collections.amazon.aws.plugins.module_utils.waiters import wait_for_resource_state


def wait_for_status(client, module: AnsibleAWSModule, waiter_name: str, nat_gateway_id: str) -> None:
    wait_timeout = module.params.get("wait_timeout")
    wait_delay = 5
    attempts = 1 + int(wait_timeout / wait_delay)
    wait_for_resource_state(
        client, module, waiter_name, delay=wait_delay, max_attempts=attempts, NatGatewayIds=[nat_gateway_id]
    )


def get_nat_gateways(
    client, subnet_id: Optional[str] = None, nat_gateway_id: Optional[str] = None, states: Optional[List[str]] = None
) -> List[Dict[str, Any]]:
    """Retrieve a list of NAT Gateways
    Args:
        client (botocore.client.EC2): Boto3 client
        module: AnsibleAWSModule class instance

    Kwargs:
        subnet_id (str): The subnet_id the nat resides in.
        nat_gateway_id (str): The Amazon NAT id.
        states (list): States available (pending, failed, available, deleting, and deleted)
            default=None

    Basic Usage:
        >>> client = boto3.client('ec2')
        >>> module = AnsibleAWSModule(...)
        >>> subnet_id = 'subnet-12345678'
        >>> get_nat_gateways(client, module, subnet_id)
        [
            {
                "create_time": "2016-03-05T00:33:21.209000+00:00",
                "delete_time": "2016-03-05T00:36:37.329000+00:00",
                "nat_gateway_addresses": [
                    {
                        "public_ip": "55.55.55.55",
                        "network_interface_id": "eni-1234567",
                        "private_ip": "10.0.0.102",
                        "allocation_id": "eipalloc-1234567"
                    }
                ],
                "nat_gateway_id": "nat-123456789",
                "state": "deleted",
                "subnet_id": "subnet-123456789",
                "tags": {},
                "vpc_id": "vpc-12345678"
            }
        ]

    Returns:
        list
    """

    params: dict[str, Any] = {}
    if not states:
        states = ["available", "pending"]
    if nat_gateway_id:
        params["NatGatewayIds"] = [nat_gateway_id]
    else:
        params["Filter"] = [
            {"Name": "subnet-id", "Values": [subnet_id]},
            {"Name": "state", "Values": states},
        ]

    return [camel_dict_to_snake_dict(gw) for gw in describe_nat_gateways(client, **params) or []]


def gateway_in_subnet_exists(
    client, subnet_id: str, allocation_id: Optional[str] = None
) -> Tuple[List[Dict[str, Any]], bool]:
    """Retrieve all NAT Gateways for a subnet.
    Args:
        client (botocore.client.EC2): Boto3 client
        module: AnsibleAWSModule class instance
        subnet_id (str): The subnet_id the nat resides in.

    Kwargs:
        allocation_id (str): The EIP Amazon identifier.
            default = None

    Basic Usage:
        >>> client = boto3.client('ec2')
        >>> module = AnsibleAWSModule(...)
        >>> subnet_id = 'subnet-1234567'
        >>> allocation_id = 'eipalloc-1234567'
        >>> gateway_in_subnet_exists(client, module, subnet_id, allocation_id)
        (
            [
                {
                    "create_time": "2016-03-05T00:33:21.209000+00:00",
                    "delete_time": "2016-03-05T00:36:37.329000+00:00",
                    "nat_gateway_addresses": [
                        {
                            "public_ip": "55.55.55.55",
                            "network_interface_id": "eni-1234567",
                            "private_ip": "10.0.0.102",
                            "allocation_id": "eipalloc-1234567"
                        }
                    ],
                    "nat_gateway_id": "nat-123456789",
                    "state": "deleted",
                    "subnet_id": "subnet-123456789",
                    "tags": {},
                    "vpc_id": "vpc-1234567"
                }
            ],
            False
        )

    Returns:
        Tuple (list, bool)
    """

    states = ["available", "pending"]
    nat_gateways = get_nat_gateways(client, subnet_id=subnet_id, states=states)
    if not allocation_id:
        return nat_gateways, False

    allocation_id_exists = False
    gateways = []
    for gw in nat_gateways:
        for address in gw["nat_gateway_addresses"]:
            if address.get("allocation_id") == allocation_id:
                allocation_id_exists = True
                gateways.append(gw)

    return gateways, allocation_id_exists


def get_eip_allocation_id_by_address(client, eip_address: str) -> Tuple[Optional[str], str]:
    """Release an EIP from your EIP Pool
    Args:
        client (botocore.client.EC2): Boto3 client
        eip_address (str): The Elastic IP Address of the EIP.

    Basic Usage:
        >>> client = boto3.client('ec2')
        >>> module = AnsibleAWSModule(...)
        >>> eip_address = '52.87.29.36'
        >>> get_eip_allocation_id_by_address(client, eip_address)
        (
            'eipalloc-36014da3', ''
        )

    Returns:
        Tuple (str, str)
    """

    params = {
        "PublicIps": [eip_address],
    }
    allocation_id = None
    msg = ""

    try:
        allocations = describe_addresses(client, **params)
        if not allocations:
            msg = f"EIP {eip_address} does not exist"
            allocation_id = None
            return allocation_id, msg

        if len(allocations) == 1:
            allocation = allocations[0]
        else:
            allocation = None

        if allocation:
            if allocation.get("Domain") != "vpc":
                msg = f"EIP {eip_address} is a non-VPC EIP, please allocate a VPC scoped EIP"
            else:
                allocation_id = allocation.get("AllocationId")

    except is_ansible_aws_error_code("InvalidAddress.Malformed"):
        raise AnsibleAWSError(message=f"EIP address {eip_address} is invalid.")

    return allocation_id, msg


def allocate_eip_address(client, check_mode: bool) -> Tuple[bool, str, Optional[str]]:
    """Release an EIP from your EIP Pool
    Args:
        client (botocore.client.EC2): Boto3 client
        module: AnsibleAWSModule class instance

    Basic Usage:
        >>> client = boto3.client('ec2')
        >>> module = AnsibleAWSModule(...)
        >>> allocate_eip_address(client, module)
        (
            True, '', ''
        )

    Returns:
        Tuple (bool, str, str)
    """

    new_eip = None
    msg = ""
    params = {
        "Domain": "vpc",
    }

    if check_mode:
        ip_allocated = True
        new_eip = None
        return ip_allocated, msg, new_eip

    new_eip = allocate_address(client, **params)["AllocationId"]
    ip_allocated = True
    msg = f"eipalloc id {new_eip} created"

    return ip_allocated, msg, new_eip


def release_eip_address(client, allocation_id: str, check_mode: bool) -> bool:
    """Release an EIP from your EIP Pool
    Args:
        client (botocore.client.EC2): Boto3 client
        module: AnsibleAWSModule class instance
        allocation_id (str): The eip Amazon identifier.

    Basic Usage:
        >>> client = boto3.client('ec2')
        >>> module = AnsibleAWSModule(...)
        >>> allocation_id = "eipalloc-123456"
        >>> release_address(client, module, allocation_id)
        True

    Returns:
        bool
    """

    changed = False
    if check_mode:
        return True

    addresses = describe_addresses(client, AllocationIds=[allocation_id])
    if addresses:
        # Release IP address
        changed = release_address(client, allocation_id=allocation_id)

    return changed


def create(client, module: AnsibleAWSModule, allocation_id: Optional[str]) -> Tuple[bool, str, Dict[str, Any]]:
    """Create an Amazon NAT Gateway.
    Args:
        client (botocore.client.EC2): Boto3 client
        module: AnsibleAWSModule class instance
        allocation_id (str): The eip Amazon identifier

    Basic Usage:
        >>> client = boto3.client('ec2')
        >>> module = AnsibleAWSModule(...)
        >>> create(client, module, allocation_id)
        [
            true,
            {
                "create_time": "2016-03-05T00:33:21.209000+00:00",
                "delete_time": "2016-03-05T00:36:37.329000+00:00",
                "nat_gateway_addresses": [
                    {
                        "public_ip": "55.55.55.55",
                        "network_interface_id": "eni-1234567",
                        "private_ip": "10.0.0.102",
                        "allocation_id": "eipalloc-1234567"
                    }
                ],
                "nat_gateway_id": "nat-123456789",
                "state": "deleted",
                "subnet_id": "subnet-1234567",
                "tags": {},
                "vpc_id": "vpc-1234567"
            },
            ""
        ]

    Returns:
        Tuple (bool, str, list)
    """

    connectivity_type = module.params.get("connectivity_type")
    wait = module.params.get("wait")
    client_token = module.params.get("client_token")
    subnet_id = module.params.get("subnet_id")
    tags = module.params.get("tags")

    params = {"SubnetId": subnet_id, "ConnectivityType": connectivity_type}

    if connectivity_type == "public":
        params.update({"AllocationId": allocation_id})

    request_time = datetime.datetime.now(datetime.timezone.utc)
    changed = False
    token_provided = False
    result: dict[str, Any] = {}
    msg = ""

    if client_token:
        token_provided = True
        params["ClientToken"] = client_token

    if tags:
        params["TagSpecifications"] = boto3_tag_specifications(tags, ["natgateway"])

    if module.check_mode:
        changed = True
        return changed, msg, result

    try:
        result = create_nat_gateway(client, **params)
        result = camel_dict_to_snake_dict(result)
        changed = True

        create_time = result["create_time"].replace(tzinfo=None)

        if token_provided and (request_time > create_time):
            changed = False

        elif wait and result.get("state") != "available":
            wait_for_status(client, module, "nat_gateway_available", result["nat_gateway_id"])

            # Get new result
            result = camel_dict_to_snake_dict(
                describe_nat_gateways(client, NatGatewayIds=[result["nat_gateway_id"]])[0]
            )

    except is_ansible_aws_error_code("IdempotentParameterMismatch") as e:
        msg = "NAT Gateway does not support update and token has already been provided:" + e
        changed = False
        result = {}

    if result:
        result["tags"] = describe_ec2_tags(client, module, result["nat_gateway_id"], resource_type="natgateway")

    return changed, msg, result


def pre_create(
    client, module: AnsibleAWSModule, allocation_id: Optional[str], eip_address: Optional[str]
) -> Tuple[bool, str, Dict[str, Any]]:
    """Create an Amazon NAT Gateway.
    Args:
        client (botocore.client.EC2): Boto3 client
        module: AnsibleAWSModule class instance

    Basic Usage:
        >>> client = boto3.client('ec2')
        >>> module = AnsibleAWSModule(...)
        >>> pre_create(client, module)
        [
            true,
            "",
            {
                "create_time": "2016-03-05T00:33:21.209000+00:00",
                "delete_time": "2016-03-05T00:36:37.329000+00:00",
                "nat_gateway_addresses": [
                    {
                        "public_ip": "52.87.29.36",
                        "network_interface_id": "eni-5579742d",
                        "private_ip": "10.0.0.102",
                        "allocation_id": "eipalloc-36014da3"
                    }
                ],
                "nat_gateway_id": "nat-03835afb6e31df79b",
                "state": "deleted",
                "subnet_id": "subnet-w4t12897",
                "tags": {},
                "vpc_id": "vpc-w68571b5"
            }
        ]

    Returns:
        Tuple (bool, str, list)
    """

    changed = False
    msg = ""
    results = {}

    connectivity_type = module.params.get("connectivity_type")
    if_exist_do_not_create = module.params.get("if_exist_do_not_create")
    purge_tags = module.params.get("purge_tags")
    default_create = module.params.get("default_create")
    subnet_id = module.params.get("subnet_id")
    tags = module.params.get("tags")

    if not allocation_id and not eip_address:
        existing_gateways, allocation_id_exists = gateway_in_subnet_exists(client, subnet_id=subnet_id)
        if len(existing_gateways) > 0 and if_exist_do_not_create:
            results = existing_gateways[0]
            changed |= ensure_ec2_tags(
                client, module, results["nat_gateway_id"], resource_type="natgateway", tags=tags, purge_tags=purge_tags
            )

            results["tags"] = describe_ec2_tags(client, module, results["nat_gateway_id"], resource_type="natgateway")

            if changed:
                return changed, msg, results

            changed = False
            msg = f"NAT Gateway {existing_gateways[0]['nat_gateway_id']} already exists in subnet_id {subnet_id}"
            return changed, msg, results
        else:
            if connectivity_type == "public":
                changed, msg, allocation_id = allocate_eip_address(client, module.check_mode)

                if not changed:
                    return changed, msg, {}

    elif eip_address or allocation_id:
        if eip_address and not allocation_id:
            allocation_id, msg = get_eip_allocation_id_by_address(client, eip_address)
            if not allocation_id and not default_create:
                changed = False
                module.fail_json(msg=msg)
            elif not allocation_id and default_create:
                eip_address = None
                return pre_create(client, module, allocation_id, eip_address)

        existing_gateways, allocation_id_exists = gateway_in_subnet_exists(
            client, subnet_id=subnet_id, allocation_id=allocation_id
        )

        if len(existing_gateways) > 0 and (allocation_id_exists or if_exist_do_not_create):
            results = existing_gateways[0]
            changed |= ensure_ec2_tags(
                client, module, results["nat_gateway_id"], resource_type="natgateway", tags=tags, purge_tags=purge_tags
            )

            results["tags"] = describe_ec2_tags(client, module, results["nat_gateway_id"], resource_type="natgateway")

            if changed:
                return changed, msg, results

            changed = False
            msg = f"NAT Gateway {existing_gateways[0]['nat_gateway_id']} already exists in subnet_id {subnet_id}"
            return changed, msg, results

    return create(client, module, allocation_id)


def remove(client, module: AnsibleAWSModule) -> Tuple[bool, str, Dict[str, Any]]:
    """Delete an Amazon NAT Gateway.
    Args:
        client (botocore.client.EC2): Boto3 client
        module: AnsibleAWSModule class instance
    Kwargs:
        wait (bool): Wait for the nat to be in the deleted state before returning.
        release_eip (bool): Once the nat has been deleted, you can deallocate the eip from the vpc.
        connectivity_type (str): private/public connection type

    Basic Usage:
        >>> client = boto3.client('ec2')
        >>> module = AnsibleAWSModule(...)
        >>> nat_gw_id = 'nat-03835afb6e31df79b'
        >>> remove(client, module, nat_gw_id, wait=True, release_eip=True, connectivity_type='public')
        [
            true,
            "",
            {
                "create_time": "2016-03-05T00:33:21.209000+00:00",
                "delete_time": "2016-03-05T00:36:37.329000+00:00",
                "nat_gateway_addresses": [
                    {
                        "public_ip": "52.87.29.36",
                        "network_interface_id": "eni-5579742d",
                        "private_ip": "10.0.0.102",
                        "allocation_id": "eipalloc-36014da3"
                    }
                ],
                "nat_gateway_id": "nat-03835afb6e31df79b",
                "state": "deleted",
                "subnet_id": "subnet-w4t12897",
                "tags": {},
                "vpc_id": "vpc-w68571b5"
            }
        ]

    Returns:
        Tuple (bool, str, list)
    """

    allocation_id = None
    changed = False
    results: dict[str, Any] = {}
    states = ["pending", "available"]
    msg = ""
    nat_gateway_id = module.params.get("nat_gateway_id")
    wait = module.params.get("wait")
    release_eip = module.params.get("release_eip")
    connectivity_type = module.params.get("connectivity_type")

    if module.check_mode:
        changed = True
        return changed, msg, results

    gw_list = get_nat_gateways(client, nat_gateway_id=nat_gateway_id, states=states)

    if len(gw_list) == 1:
        results = gw_list[0]
        try:
            changed = delete_nat_gateway(client, nat_gateway_id)
        except AnsibleEC2Error as e:
            module.fail_json_aws_error(e)
        if connectivity_type == "public":
            allocation_id = results["nat_gateway_addresses"][0]["allocation_id"]
        msg = f"NAT gateway {nat_gateway_id} is in a deleting state. Delete was successful"

        if wait and results.get("state") != "deleted":
            wait_for_status(client, module, "nat_gateway_deleted", nat_gateway_id)

            # Get new results
            results = camel_dict_to_snake_dict(describe_nat_gateways(client, NatGatewayIds=[nat_gateway_id])[0])
            results["tags"] = describe_ec2_tags(client, module, nat_gateway_id, resource_type="natgateway")

    if release_eip and allocation_id:
        changed |= release_eip_address(client, allocation_id, module.check_mode)

    return changed, msg, results


def main() -> None:
    argument_spec = dict(
        subnet_id=dict(type="str"),
        eip_address=dict(type="str"),
        allocation_id=dict(type="str"),
        connectivity_type=dict(type="str", default="public", choices=["private", "public"]),
        if_exist_do_not_create=dict(type="bool", default=False),
        state=dict(default="present", choices=["present", "absent"]),
        wait=dict(type="bool", default=False),
        wait_timeout=dict(type="int", default=320, required=False),
        release_eip=dict(type="bool", default=False),
        nat_gateway_id=dict(type="str"),
        client_token=dict(type="str", no_log=False),
        tags=dict(required=False, type="dict", aliases=["resource_tags"]),
        purge_tags=dict(default=True, type="bool"),
        default_create=dict(type="bool", default=False),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        mutually_exclusive=[["allocation_id", "eip_address"]],
        required_if=[["state", "absent", ["nat_gateway_id"]], ["state", "present", ["subnet_id"]]],
    )

    try:
        client = module.client("ec2")
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to connect to AWS.")

    changed = False
    msg = ""
    state = module.params.get("state")

    try:
        if state == "present":
            allocation_id = module.params.get("allocation_id")
            eip_address = module.params.get("eip_address")
            changed, msg, results = pre_create(client, module, allocation_id, eip_address)
        else:
            changed, msg, results = remove(client, module)
    except AnsibleAWSError as e:
        module.fail_json_aws_error(e)

    module.exit_json(msg=msg, changed=changed, **results)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2018, Micah Hunsberger (@mhunsber)
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r'''
---
module: win_hosts
short_description: Manages hosts file entries on Windows.
description:
  - Manages hosts file entries on Windows.
  - Maps IPv4 or IPv6 addresses to canonical names.
  - Adds, removes, or sets cname records for ip and hostname pairs.
  - Modifies %windir%\\system32\\drivers\\etc\\hosts.
version_added: 2.6.0
options:
  state:
    description:
      - Whether the entry should be present or absent.
      - If only I(canonical_name) is provided when C(state=absent), then
        all hosts entries with the canonical name of I(canonical_name)
        will be removed.
      - If only I(ip_address) is provided when C(state=absent), then all
        hosts entries with the ip address of I(ip_address) will be removed.
      - If I(ip_address) and I(canonical_name) are both omitted when
        C(state=absent), then all hosts entries will be removed.
    choices:
      - absent
      - present
    default: present
    type: str
  canonical_name:
    description:
      - A canonical name for the host entry.
      - required for C(state=present).
    type: str
  ip_address:
    description:
      - The ip address for the host entry.
      - Can be either IPv4 (A record) or IPv6 (AAAA record).
      - Required for C(state=present).
    type: str
  aliases:
    description:
      - A list of additional names (cname records) for the host entry.
      - Only applicable when C(state=present).
    type: list
    elements: str
  action:
    choices:
      - add
      - remove
      - set
    description:
      - Controls the behavior of I(aliases).
      - Only applicable when C(state=present).
      - If C(add), each alias in I(aliases) will be added to the host entry.
      - If C(set), each alias in I(aliases) will be added to the host entry,
        and other aliases will be removed from the entry.
    default: set
    type: str
author:
  - Micah Hunsberger (@mhunsber)
notes:
  - Each canonical name can only be mapped to one IPv4 and one IPv6 address.
    If I(canonical_name) is provided with C(state=present) and is found
    to be mapped to another IP address that is the same type as, but unique
    from I(ip_address), then I(canonical_name) and all I(aliases) will
    be removed from the entry and added to an entry with the provided IP address.
  - Each alias can only be mapped to one canonical name. If I(aliases) is provided
    with C(state=present) and an alias is found to be mapped to another canonical
    name, then the alias will be removed from the entry and either added to or removed
    from (depending on I(action)) an entry with the provided canonical name.
seealso:
  - module: ansible.windows.win_template
  - module: ansible.windows.win_file
  - module: ansible.windows.win_copy
'''

EXAMPLES = r'''
- name: Add 127.0.0.1 as an A record for localhost
  ansible.windows.win_hosts:
    state: present
    canonical_name: localhost
    ip_address: 127.0.0.1

- name: Add ::1 as an AAAA record for localhost
  ansible.windows.win_hosts:
    state: present
    canonical_name: localhost
    ip_address: '::1'

- name: Remove 'bar' and 'zed' from the list of aliases for foo (192.168.1.100)
  ansible.windows.win_hosts:
    state: present
    canonical_name: foo
    ip_address: 192.168.1.100
    action: remove
    aliases:
      - bar
      - zed

- name: Remove hosts entries with canonical name 'bar'
  ansible.windows.win_hosts:
    state: absent
    canonical_name: bar

- name: Remove 10.2.0.1 from the list of hosts
  ansible.windows.win_hosts:
    state: absent
    ip_address: 10.2.0.1

- name: Ensure all name resolution is handled by DNS
  ansible.windows.win_hosts:
    state: absent
'''

RETURN = r'''
'''



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_assurance_alerts_overview_by_type_info
short_description: Information module for organizations _assurance _alerts _overview
  _by _type
description:
  - Get all organizations _assurance _alerts _overview _by _type.
  - Return a Summary of Alerts grouped by type and severity.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - OrganizationId path parameter. Organization ID.
    type: str
  perPage:
    description:
      - PerPage query parameter. The number of entries per page returned. Acceptable
        range is 3 - 1000. Default is 1000.
    type: int
  startingAfter:
    description:
      - >
        StartingAfter query parameter. A token used by the server to indicate the
        start of the page. Often this is a timestamp or an ID but it is not limited
        to those. This parameter should not be defined by client applications. The
        link for the first, last, prev, or next page in the HTTP Link header should
        define it.
    type: str
  endingBefore:
    description:
      - >
        EndingBefore query parameter. A token used by the server to indicate the end
        of the page. Often this is a timestamp or an ID but it is not limited to those.
        This parameter should not be defined by client applications. The link for
        the first, last, prev, or next page in the HTTP Link header should define
        it.
    type: str
  sortOrder:
    description:
      - >
        SortOrder query parameter. Sorted order of entries. Order options are 'ascending'
        and 'descending'. Default is 'ascending'.
    type: str
  networkId:
    description:
      - NetworkId query parameter. Optional parameter to filter alerts overview by
        network ids.
    type: str
  severity:
    description:
      - Severity query parameter. Optional parameter to filter alerts overview by
        severity type.
    type: str
  types:
    description:
      - Types query parameter. Optional parameter to filter by alert type.
    elements: str
    type: list
  tsStart:
    description:
      - TsStart query parameter. Optional parameter to filter by starting timestamp.
    type: str
  tsEnd:
    description:
      - TsEnd query parameter. Optional parameter to filter by end timestamp.
    type: str
  category:
    description:
      - Category query parameter. Optional parameter to filter by category.
    type: str
  sortBy:
    description:
      - SortBy query parameter. Optional parameter to set column to sort by.
    type: str
  serials:
    description:
      - Serials query parameter. Optional parameter to filter by primary device serial.
    elements: str
    type: list
  deviceTypes:
    description:
      - DeviceTypes query parameter. Optional parameter to filter by device types.
    elements: str
    type: list
  deviceTags:
    description:
      - DeviceTags query parameter. Optional parameter to filter by device tags.
    elements: str
    type: list
  active:
    description:
      - Active query parameter. Optional parameter to filter by active alerts defaults
        to true.
    type: bool
  dismissed:
    description:
      - Dismissed query parameter. Optional parameter to filter by dismissed alerts
        defaults to false.
    type: bool
  resolved:
    description:
      - Resolved query parameter. Optional parameter to filter by resolved alerts
        defaults to false.
    type: bool
  suppressAlertsForOfflineNodes:
    description:
      - >
        SuppressAlertsForOfflineNodes query parameter. When set to true the api will
        only return connectivity alerts for a given device if that device is in an
        offline state. This only applies to devices. This is ignored when resolved
        is true. Example If a Switch has a VLan Mismatch and is Unreachable. Only
        the Unreachable alert will be returned. Defaults to false.
    type: bool
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations getOrganizationAssuranceAlertsOverviewByType
    description: Complete reference of the getOrganizationAssuranceAlertsOverviewByType
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-assurance-alerts-overview-by-type
notes:
  - SDK Method used are
    organizations.Organizations.get_organization_assurance_alerts_overview_by_type,
  - Paths used are
    get /organizations/{organizationId}/assurance/alerts/overview/byType,
"""

EXAMPLES = r"""
- name: Get all organizations _assurance _alerts _overview _by _type
  cisco.meraki.organizations_assurance_alerts_overview_by_type_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    perPage: 0
    startingAfter: string
    endingBefore: string
    sortOrder: string
    networkId: string
    severity: string
    types: []
    tsStart: string
    tsEnd: string
    category: string
    sortBy: string
    serials: []
    deviceTypes: []
    deviceTags: []
    active: true
    dismissed: true
    resolved: true
    suppressAlertsForOfflineNodes: true
    organizationId: string
    total_pages: -1
    direction: next
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "items": [
        {
          "type": "string",
          "lastResolvedAt": "string",
          "count": 0
        }
      ],
      "meta": {
        "counts": {
          "items": 0
        }
      }
    }
"""
